/***************************************************************************
                          centralwin.cpp  -  description
                             -------------------
    begin                : Mit Mr 5 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "centralwin.h"
#include "renametabdlg.h"
#include <klocale.h>
#include <qpoint.h>
#include <qrect.h>
#include <qtabbar.h>
#include <qiconset.h>
#include <qpixmap.h>

CentralWin::CentralWin(QWidget *parent, const char *name ) : QTabWidget(parent) {
  if(parent){
    parent->installEventFilter(this);
  }

  QPixmap plugPix =  iLoader.loadIcon("unplug", KIcon::Small);
  QPixmap delTabPix =  iLoader.loadIcon("del_tab", KIcon::Small);
  tabMenu = new QPopupMenu();
  tabMenu->insertItem(QIconSet(plugPix), i18n("unplug"));
  tabMenu->insertItem(i18n("rename"));
  tabMenu->insertItem(QIconSet(delTabPix), i18n("close"));
  connect(tabMenu, SIGNAL(activated(int)), this, SLOT(slotAction(int)));
  
  //showStartTab();
  queryList = false;
  currentLabel = "";
}
CentralWin::~CentralWin(){
}
void CentralWin::showStartTab(){
   CwinStartTab *tab = new CwinStartTab("start", this);
   addTab(tab, "start");
}
QString CentralWin::makeLabel(const QString & preLabel){
  int tabNum = 1;
  QString label;
  
  label = preLabel + QString(" %1").arg(tabNum);

  labelMapIt = labelMap.find(label);

  while(labelMapIt != labelMap.end()){
    tabNum++;
    label = preLabel + QString(" %1").arg(tabNum);
    labelMapIt = labelMap.find(label);
  }

  return label;
}
void CentralWin::showResultTab(KSqlShellDoc *doc, SqlClient *sqlCl){

  if(sqlCl->queryInput.isRefresh()){
    //qDebug(sqlCl->queryInput.getResultLabel().latin1());
    CwinResultTab *rTab = (CwinResultTab *)labelMap[sqlCl->queryInput.getResultLabel()];
    rTab->refresh(sqlCl);
    if(rTab->isPlug()){
      showPage(rTab);
    }else{
      rTab->raise();
    }
    return;
  }
  
  Result * res;
  
  for(uint i = 0; i < sqlCl->resultList.count();){
    
    res = sqlCl->resultList.at(i);
    
    QString label;

    if(!res->stmtOk())
        label = makeLabel("err");
    else
        label = makeLabel("res");

    CwinSqlOutTab *tab;
    
    if(!res->stmtOk()){
      tab = new CwinErrorTab(doc, sqlCl, res, label, this);

    }else if(sqlCl->queryInput.getType() == QueryInput::Free && res->isResult()){
      tab = new CwinResultTab(doc, sqlCl, res, label, this);

    }else if(sqlCl->queryInput.getType() == QueryInput::Select){
      tab = new CwinSelAllResTab(doc, sqlCl, res, label, this);
    }else{
      sqlCl->resultList.remove(res);  
      continue;
    }
    
    labelMap[label] = tab;
    addTab(tab, label);
    showPage(tab);
    
    if(!res->stmtOk())
      return;

    i++;
  }    
}
void CentralWin::showTableTab(KSqlShellDoc *doc, SqlTable & sqlTable){

  QString label = makeLabel("tab");
  
  CwinTableTab *tab = new CwinTableTab(doc, sqlTable, label, this);
  labelMap[label] = tab;
  addTab(tab, label);
  showPage(tab);
}
void CentralWin::showInfoTab(KSqlShellDoc * doc, const QString & conStr){
  QString label = makeLabel("info");

   CwinInfoTab *tab = new CwinInfoTab(doc, conStr, label, this);
   labelMap[label] = tab;
   addTab(tab, label);
   showPage(tab);
}
void CentralWin::showQueryList(KSqlShellDoc *doc){
  if(queryList){
    CwinTab *tab = 0;
    tab = labelMap[i18n("query list")];
    if(!tab)
      return;
    if(tab->isPlug()){
      showPage(tab);
    }else{
      tab->raise();
    }
    return;
  }

  QString label = i18n("query list");
  qListTab = new CwinQueryListTab(doc, label, this);
  labelMap[label] = qListTab;
  addTab(qListTab, label);
  showPage(qListTab);
  queryList = true;
}
bool CentralWin::eventFilter(QObject *o, QEvent *e){
  if(e->type() == QEvent::MouseButtonPress){
    QMouseEvent *m = (QMouseEvent *)e;
    if(m->button() == Qt::RightButton){

      QPoint point = m->pos();

      QTabBar * tabB = tabBar();
      if(tabB){
        QTab *tab = tabB->selectTab(point);
        if(tab){
          currentLabel = tab->text();
          
          if(currentLabel == i18n("query list"))
            tabMenu->setItemEnabled(tabMenu->idAt(1), false);
          else
            tabMenu->setItemEnabled(tabMenu->idAt(1), true);
            
          tabMenu->popup(mapToGlobal(point));                  
          return true;
        }
      }
      
    }
  }
  return QTabWidget::eventFilter(o, e);
}
void CentralWin::slotAction(int id){
  int index = tabMenu->indexOf(id);
  if(index == 0){
    if(labelMap.contains(currentLabel)){
      labelMap[currentLabel]->slotPlugTab();
    }
  }else if(index == 1){
    
    RenameTabDlg dlg(this, i18n("rename"));
    dlg.exec();

  }else if(index == 2){
    if(labelMap.contains(currentLabel)){
      labelMap[currentLabel]->slotDelTab();
    }
  }
}