/***************************************************************************
                          clqueue.cpp  -  description
                             -------------------
    begin                : Die Mr 18 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <klocale.h>
#include "clqueue.h"

ClQueue::ClQueue(){
}
ClQueue::~ClQueue(){
}

bool ClQueue::initQueue(DbType dbType,
                                      const QString & dbHost,
                                      int dbPort,
                                      const QString & dbUser,
                                      const QString & dbPw,
                                      const QString & dbName)
{
  type = dbType;
  host = dbHost;
  port = dbPort;
  user = dbUser;
  pw = dbPw;
  defDb = dbName;
  error = "";
  defStackSize = 0;
  sqlClQueueIndex = 0;
  isCon = false;
  sqlClStack.setAutoDelete(true);
  versionStr = "";
  versionNum = 0.0;       
  return true;
}


bool ClQueue::connect(int defConNum){

  for (int i = 0; i < defConNum; i++){
    
    SqlClient *cl;
    
    if(type == MYSQL)
      cl = new MySqlCl(host, port, user, pw, defDb);
    else if(type == PGSQL)
      cl =  new PgSqlCl(host, port, user, pw, defDb);
    else{
      error =  i18n("unknown db type");
      return false;
    }
      
    if(!cl->initSqlCl()){
      error = cl->getError();
      return false;
    }
     
    sqlClStack.push(cl);
  }

  if(versionStr.isEmpty()){
    SqlClient *sqlCl = getCl();
    if(!sqlCl)
      return false;

    if(!sqlCl->getVersionStr()){
      error = sqlCl->getError();
      return false;
    }
    
    Result *res = sqlCl->resultList.at(0);
    while(res->nextRow())
      versionStr = QString(res->getValue(0));

    versionNum = sqlCl->getVersionNum(versionStr);
        
    freeCl(sqlCl);
  }
  defStackSize = defConNum;
  isCon = true;
  return true;
}

void ClQueue::disconnect(){
  sqlClStack.clear();
  isCon = false;
  versionStr = "";
}

SqlClient * ClQueue::getCl(){
  if(sqlClStack.isEmpty()){
    if(!connect(1)){
      return 0;
    }
  }
  return sqlClStack.pop();
}

void ClQueue::freeCl(SqlClient * sqlCl){
  if((int)sqlClStack.count() <= defStackSize && isCon){
    sqlCl->clear();
    sqlClStack.push(sqlCl);   
  }else
    delete sqlCl;
}

SqlClient * ClQueue::dequeueCl(){
  return sqlClQueue.dequeue();
}

void ClQueue::enqueueCl(SqlClient *sqlCl){
  sqlClQueue.enqueue(sqlCl);
}

SqlClient * ClQueue::getResultCl(){
    
  while(sqlClQueueIndex < sqlClQueue.count()){
    SqlClient * sqlCl = sqlClQueue.dequeue();
    
    sqlClQueueIndex ++;
    
    if(sqlCl->queryEnd())
      return sqlCl;
      
    sqlClQueue.enqueue(sqlCl);
    
  }
  
  sqlClQueueIndex = 0;
  
  return 0;
}

bool ClQueue::dropDb(const QString & db){
  QString st = "drop database " + db;
  return runStmt(st);
}

bool ClQueue::isRunning(){
   return (!sqlClQueue.isEmpty());
}

QString ClQueue::getError(){
  QString err = error;
  error = "";
  return err;
}
QString ClQueue::getTypeStr(){
  return getDbTypeStr(type);
}
QString ClQueue::getHost(){
  return host;
}
QString ClQueue::getUser(){
  return user;
}
QString ClQueue::getPw(){
  return pw;
}
QString ClQueue::getDefDb(){
  return defDb;
}
QString ClQueue::getPort(){
  return QString::number(port);
}



bool ClQueue::runStmt(const QString & stmt, const QString & db){
  if(!isCon){
    error = i18n("is not connected!");
    return false;
  }
  
  SqlClient *sqlCl = getCl();
  if(!sqlCl)
    return false;
  
  if(!sqlCl->useDb(db) || !sqlCl->runStmt(stmt, false)){
    error = sqlCl->getError();
    freeCl(sqlCl);
    return false;
  }
 
  freeCl(sqlCl);

  return true;
}
bool ClQueue::sendQuery(const QueryInput & queryInput){
  if(!isCon){
    error = i18n("is not connected!");
    return false;
  }
  
  SqlClient *sqlCl = getCl();

  if(!sqlCl)
    return false;
  
  if(!sqlCl->sendQuery(queryInput)){
    error = sqlCl->getError();
    return false;
  }

  enqueueCl(sqlCl);
  
  return true;
}

bool ClQueue::getTemplates(QStrList * list){
  SqlClient *sqlCl = getCl();
  if(!sqlCl)
    return false;

  if(!sqlCl->getTemplates()){
    error = sqlCl->getError();
    freeCl(sqlCl);
    return false;
  }
  Result *res = sqlCl->resultList.at(0);
  while(res->nextRow())
    list->append(res->getValue(0));

  freeCl(sqlCl);

  return true;
}
bool ClQueue::getSqlTable(SqlTable * sqlTable){
  SqlClient *sqlCl = getCl();
  if(!sqlCl)
    return false;

  
  if(!sqlCl->getSqlTable(sqlTable)){
    error = sqlCl->getError();
    freeCl(sqlCl);
    return false;
  }
  
  freeCl(sqlCl);
  return true;
}
SqlTypes ClQueue::getSqlTypes(){
  if(type == MYSQL)
    return SqlTypes(MySqlTypes, MYSQL_TYPE_NUM);
  else
    return SqlTypes(PgSqlTypes, PGSQL_TYPE_NUM);
}
QString ClQueue::getCreateStmt(SqlTable & table){
  SqlTable::TableAction action = table.getTableAction();
  QString tabStr = table.getTabStr();
  QString dbStr = table.getDbStr();
  int fieldNum = table.getFieldNum();

  QString stmt = "";
  
  if(action == SqlTable::NewTable){
    stmt = "CREATE TABLE " + tabStr + " (";

    QStringList stmtList;

    QString tail("");

    QStringList uniqueList;
    QStringList primKeyList;
    QStringList indexList;
    QStringList fullTextList;

    for(int f = 0; f < fieldNum; f++){

      if(table.mapList["Name"][f].isEmpty())
        continue;

      QString field("");
      field += table.mapList["Name"][f] + " " + table.mapList["Type"][f];

      if(table.mapList["Type"][f] == "set" || table.mapList["Type"][f] == "enum")
        field += "(" + table.mapList["Values"][f] + ")";
      if(!table.mapList["Length, Dec"][f].isEmpty())
        field += "(" + table.mapList["Length, Dec"][f] + ")";

      if(!table.mapList["Attribute"][f].isEmpty())
        field += " " + table.mapList["Attribute"][f];

      if(!table.mapList["Default Value"][f].isEmpty())
        field += " DEFAULT '"  + table.mapList["Default Value"][f] + "'";

      if(table.mapList["Not Null"][f] == "Not Null")
        field += " NOT NULL";

      if(table.mapList.contains("Options"))
        if(table.mapList["Options"][f] == "auto_increment")
        field += " AUTO_INCREMENT";
      else if(table.mapList["Options"][f] == "fulltext")
        fullTextList.append(table.mapList["Name"][f]);

      stmtList.append(field);

      if(table.mapList["Unique"][f] == "Unique")
        uniqueList.append(table.mapList["Name"][f]);

      if(table.mapList["Primary Key"][f] == "Primary Key")
        primKeyList.append(table.mapList["Name"][f]);

      if(table.mapList.contains("Index") && table.mapList["Index"][f] == "Index")
        indexList.append(table.mapList["Name"][f]);

    }

    if(!uniqueList.isEmpty())
      stmtList.append("UNIQUE(" + uniqueList.join(",") + ")");
    if(!primKeyList.isEmpty())
      stmtList.append("PRIMARY KEY(" + primKeyList.join(",") + ")");
    if(!indexList.isEmpty())
      stmtList.append("INDEX(" + indexList.join(",") + ")");
    if(!fullTextList.isEmpty())
      stmtList.append("FULLTEXT(" + fullTextList.join(",") + ")");
    if(!table.getCheck().isEmpty())
      stmtList.append("CHECK(" + table.getCheck() + ")");

    if(!table.inheritsList.isEmpty())
      tail += " INHERITS(" + table.inheritsList.join(",") + ")";

    if(table.isOid())
      tail += " WITH OIDS";
    else if(table.isWoOid())
      tail += " WITHOUT OIDS";


    stmt += stmtList.join(",") + ")" + tail;
  }
  //qDebug(stmt.latin1());

  return stmt;
}
bool ClQueue::makeTable(SqlTable & table){

  SqlClient *sqlCl = getCl();
  if(!sqlCl)
    return false;
        
  SqlTable::TableAction action = table.getTableAction();
  QString tabStr = table.getTabStr();
  QString dbStr = table.getDbStr();
  int fieldNum = table.getFieldNum();

  QString stmt = "";

  if(action == SqlTable::NewTable){
    
    stmt = getCreateStmt(table);

    //qDebug(stmt.latin1());

  }else if(action == SqlTable::RenameTable){
    stmt = "ALTER TABLE " + tabStr + " RENAME TO " + table.getNewTabName();                
  }else if(action == SqlTable::DelTable){
    stmt = "DROP TABLE " + tabStr;   
  }else if(action == SqlTable::DelField){
    stmt = "ALTER TABLE " + tabStr + " DROP " + table.getChangeField();
  }

  if(!stmt.isEmpty()){
    if(!runStmt(stmt, dbStr)){
      freeCl(sqlCl);
      return false;
    }
  }

  if(action == SqlTable::RenameField || action == SqlTable::ChangeField || action == SqlTable::AddField){
    if(!sqlCl->useDb(dbStr) || !sqlCl->makeTable(table)){
      error = sqlCl->getError();
      freeCl(sqlCl);
      return false;
    }
  }

  freeCl(sqlCl);
  return true;
}
bool ClQueue::createDb(const QString & db, const QString & templateDb){
  
  SqlClient *sqlCl = getCl();
  if(!sqlCl)
    return false;

  if(!sqlCl->createDb(db, templateDb)){
    error = sqlCl->getError();
    freeCl(sqlCl);
    return false;
  }

  freeCl(sqlCl);
  return true;
}
