/***************************************************************************
                          clqueue.h  -  description
                             -------------------
    begin                : Die Mr 18 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CLQUEUE_H
#define CLQUEUE_H

#include <qstring.h>
#include <qptrstack.h>
#include <qptrqueue.h>
#include <mysqlcl.h>
#include <pgsqlcl.h>
#include <qstrlist.h>
#include <qmap.h>
#include <qstringlist.h>
#include "sqlclient.h"
#include "queryinput.h"
#include "sqltable.h"


/**
  *@author Olaf Gawenda
  */

class ClQueue{
public:
    enum DbType { MYSQL, PGSQL };
    ClQueue();
	bool initQueue(DbType dbType,
                      const QString & dbHost,
                      int dbPort, const QString & dbUser,
                      const QString & dbPw,
                      const QString & dbName);
                     
	~ClQueue();
    bool connect(int defConNum);
    void disconnect();
    SqlClient * getCl();
    void freeCl(SqlClient * sqlCl);
    SqlClient * dequeueCl();
    void enqueueCl(SqlClient * sqlCl);
    SqlClient * getResultCl();
    DbType getType(){
      return type;
    }
    QString getError();
    QString getTypeStr();
    QString getVersionStr(){return versionStr;};
    float getVersionNum(){return versionNum;};
    QString getHost();
    QString getPort();
    QString getUser();
    QString getPw();
    QString getDefDb();
    bool isConnect(){
      return isCon;
    }
    bool isRunning();
    
    static QString getDbTypeStr(DbType t){
      if(t == MYSQL)
        return QString("MySql");
      else if(t == PGSQL)
        return QString("PgSql");

      return QString("");
    }
    static DbType getDbType(QString t){
      if(t == "MySql")
        return MYSQL;
      else
        return PGSQL;
    }

    bool getTemplates(QStrList * list);
    bool getSqlTable(SqlTable * sqlTable);
    SqlTypes getSqlTypes();
    bool makeTable(SqlTable & table);
    bool createDb(const QString & db, const QString & templateDb = "");
    bool dropDb(const QString & db);
    bool runStmt(const QString & stmt, const QString & db = "");
    bool sendQuery(const QueryInput & queryInupt);
    QString getCreateStmt(SqlTable & table);
    
private:
    DbType type;
    QString host;
    QString user;
    QString pw;
    QString defDb;
    QString versionStr;
    float versionNum;
    QString error;
    int port;
    int defStackSize;
    bool isCon;
    QPtrStack<SqlClient> sqlClStack;
    uint sqlClQueueIndex;
    QPtrQueue<SqlClient> sqlClQueue;
};

#endif
