/***************************************************************************
                          querylisttab.cpp  -  description
                             -------------------
    begin                : Fre Jun 20 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <klocale.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <kmessagebox.h>
#include "cwinquerylisttab.h"

CwinQueryListTab::CwinQueryListTab(KSqlShellDoc * document, const QString & label, QWidget *parent, const char *name ) : CwinTab(label, parent,name) {
  saveQueryPix =  iLoader.loadIcon("filesave", KIcon::MainToolbar);
  newConPix = iLoader.loadIcon("filenew", KIcon::MainToolbar);
  doc = document;
  cwin = (CentralWin*)parent;
  delete headerPage;
  cut = false;
  cItem = 0;
  
  plugButton = new QPushButton(QIconSet(unplugTabPix),"", page);
  delButton = new QPushButton(QIconSet(delTabPix),"", page);
  connect(plugButton, SIGNAL(clicked()), this, SLOT(slotPlugTab()));
  connect(delButton, SIGNAL(clicked()), this, SLOT(slotDelTab()));
  QToolTip::add(plugButton, i18n("unplug"));
  QToolTip::add(delButton, i18n("close"));
  
  saveButton = new QPushButton(QIconSet(saveQueryPix),"",page);
  newConButton = new QPushButton(QIconSet(newConPix),"",page);
  connect(saveButton, SIGNAL(clicked()), this, SLOT(slotSaveList()));
  connect(newConButton, SIGNAL(clicked()), this, SLOT(slotNewCon()));
  QToolTip::add(saveButton, i18n("save query"));
  QToolTip::add(newConButton, i18n("New Connection"));
  
  hLayout->insertStretch(0);
  hLayout->addWidget(newConButton);
  hLayout->addWidget(saveButton);
  hLayout->addWidget(plugButton);
  hLayout->addWidget(delButton);
  
  hLayout2 = new QHBoxLayout(layout);

  vGroupBox = new QVGroupBox(i18n("Query List"), page);
  queryList = new NaviList(vGroupBox);
  queryList->removeColumn(1);
  connect(queryList, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(slotSelect(QListViewItem *)));
  connect(queryList, SIGNAL(itemRenamed(QListViewItem *, int, const QString &)), this, SLOT(slotRename(QListViewItem *, int, const QString &)));
  connect(queryList ,SIGNAL(contextMenuRequested(QListViewItem*, const QPoint &, int)), this, SLOT(slotListPopupMenu(QListViewItem*, const QPoint &, int)));
  hLayout2->addWidget(vGroupBox);

  vLayout2 = new QVBoxLayout(hLayout2);
  
  vGroupBox1 = new QVGroupBox(i18n("Query"), page);
  queryText = new QTextEdit(vGroupBox1);
  vLayout2->addWidget(vGroupBox1);
  vGroupBox1->setMinimumWidth(450);

  vGroupBox2 = new QVGroupBox(i18n("Discribtion"), page);
  queryDiscr = new QTextEdit(vGroupBox2);
  vLayout2->addWidget(vGroupBox2);
  vGroupBox2->setMinimumWidth(450);

  conMenu = new QPopupMenu();
  conMenu->insertItem(i18n("new"));
  conMenu->insertItem(i18n("rename"));
  conMenu->insertItem(i18n("delete"));
  connect(conMenu, SIGNAL(activated(int)),this, SLOT(slotConMenuActivated(int)));

  dbMenu = new QPopupMenu();
  dbMenu->insertItem(i18n("new"));
  dbMenu->insertItem(i18n("paste query"));
  dbMenu->insertItem(i18n("rename"));
  dbMenu->insertItem(i18n("delete"));
  connect(dbMenu, SIGNAL(activated(int)),this, SLOT(slotDbMenuActivated(int)));
  
  queryMenu = new QPopupMenu();
  queryMenu->insertItem(i18n("run"));
  queryMenu->insertSeparator();
  queryMenu->insertItem(i18n("copy"));
  queryMenu->insertItem(i18n("cut"));
  queryMenu->insertSeparator();
  queryMenu->insertItem(i18n("rename"));
  queryMenu->insertItem(i18n("delete"));
  
  connect(queryMenu, SIGNAL(activated(int)),this, SLOT(slotQueryMenuActivated(int)));
  
  slotSyncList();
}
CwinQueryListTab::~CwinQueryListTab(){
}
void CwinQueryListTab::delTab(){
  cwin->queryList = false;
}
void CwinQueryListTab::sync(QStringList & list, QListViewItem * item, bool renameEnabled){
  QListViewItem *i, *ni, *newI;
  QStringList::Iterator it;
  
  if(item)
    i = item->firstChild();
  else
    i = queryList->firstChild();
  
  while(i){
    ni = i->nextSibling();
    it = list.find(i->text(0));
    if(it != list.end()){
      list.remove(it);
    }else{
      if(item){
        item->takeItem(i);
      }else{
        queryList->takeItem(i);
      }
      delete i;
    }
    i = ni;
  }

  for(it = list.begin(); it != list.end(); ++it){

    if(item){
      newI = new QListViewItem(item, (*it));
      
    }else{
      newI = new QListViewItem(queryList, (*it));
    }

    newI->setRenameEnabled(0, renameEnabled);
  }
}
void CwinQueryListTab::slotSyncList(){
  QStringList conList = doc->queryMap.getConNameList();  
  QListViewItem *con, *nextCon, *db, *nextDb;

  if(conList.isEmpty())
    slotSelect(0);
      
  sync(conList, 0, true);
  
  con = queryList->firstChild();

  while(con){
    QStringList dbList = doc->queryMap.getDbNameList(con->text(0));
    sync(dbList, con, true);
    db =  con->firstChild();    
    while(db){
      QStringList queryList = doc->queryMap.getQueryNameList(con->text(0), db->text(0));
      sync(queryList, db, true);
      db = db->nextSibling();
    }

    con = con->nextSibling();
    
  }

  
}

void CwinQueryListTab::slotDelText(){
  queryDiscr->clear();
  queryText->clear();
  queryList->setSelected(queryList->currentItem(), false);
}

void CwinQueryListTab::slotSaveList(){
  if(!queryList->currentItem() || queryList->currentItem()->depth() != 2)
    return;

  updateQuery(queryList->currentItem(), false);
}
void CwinQueryListTab::slotRename(QListViewItem *item, int col, const QString & text){
  if(currentItemStr == text)
    return;
  //updateQuery(item, true);

  QString con = "";
  QString db = "";
  QString query = "";
  
  if(item->depth() == 0)
    con = currentItemStr;

  if(item->depth() == 1){
    con = item->parent()->text(0);
    db = currentItemStr;
  }

  if(item->depth() == 2){
    con = item->parent()->parent()->text(0);
    db = item->parent()->text(0);
    query = currentItemStr;
  }
  
  if(!doc->queryMap.rename(item->text(0), con, db, query)){
    KMessageBox::error(this, doc->queryMap.getError());
    item->setText(0, currentItemStr);
    return;
  }
  
  
  currentItemStr = item->text(0);
}
void CwinQueryListTab::slotSelect(QListViewItem *item){
     
  if(!item || item->depth() != 2){
    queryText->setText("");
    queryDiscr->setText("");
    queryText->setReadOnly(true);
    queryDiscr->setReadOnly(true);
    if(!item)
      return;
  }else{
    QueryInput qi;
    qi.setConStr(item->parent()->parent()->text(0));
    qi.setDbStr(item->parent()->text(0));
    qi.setQueryName(item->text(0));

    doc->queryMap.getQueryInput(&qi);
    
    queryText->setReadOnly(false);
    queryDiscr->setReadOnly(false);
    
    queryText->setText(qi.getQuery());
    queryDiscr->setText(qi.getQueryText());
  }
      
  currentItemStr = item->text(0);
}
void CwinQueryListTab::slotListPopupMenu(QListViewItem* listItem, const QPoint &p, int con){

  if(!listItem)          // QListView send Signal contextMenuRequested without valid QListViewItem pointer ???
    return;

  if(cItem){
    queryMenu->setItemEnabled(queryMenu->idAt(2), false);
    queryMenu->setItemEnabled(queryMenu->idAt(3), false);
    dbMenu->setItemEnabled(dbMenu->idAt(1), true);
  }else{
    queryMenu->setItemEnabled(queryMenu->idAt(2), true);
    queryMenu->setItemEnabled(queryMenu->idAt(3), true);
    dbMenu->setItemEnabled(dbMenu->idAt(1), false);
  }
  
  if(listItem->depth() == 2){
  
    queryMenu->popup(p);
    
  }else if(listItem->depth() == 1)
    dbMenu->popup(p);
  else 
    conMenu->popup(p); 
}
QString CwinQueryListTab::getNewName(QListViewItem *item){
  QListViewItem *newItem;
  QString newItemStr = "new";
  int i = 0;
  while(1){
    
    if(!item)
      newItem = queryList->findNaviItem(newItemStr);
    else if(item->depth() == 0)
      newItem = queryList->findNaviItem(item->text(0), newItemStr);
    else if(item->depth() == 1)
      newItem = queryList->findNaviItem(item->parent()->text(0), item->text(0), newItemStr);

    if(i > 10)
      return "?????";
    if(!newItem)
      return newItemStr;
      
    i++;
    newItemStr += " new";
  }
}
void CwinQueryListTab::newItem(){
  QListViewItem *item = queryList->currentItem();
  QString newName = getNewName(item);
  
  item->setOpen(true);
  
  if(item->depth() == 0){
    if(!doc->queryMap.insertDb(item->text(0), newName)){
      KMessageBox::error(this, doc->queryMap.getError());
      return;
    }
  }else if(item->depth() == 1){
    QueryInput qi;
    qi.setConStr(item->parent()->text(0));
    qi.setDbStr(item->text(0));
    qi.setQueryName(newName);
    
    if(!doc->queryMap.insertQueryInput(&qi)){
      KMessageBox::error(this, doc->queryMap.getError());
      return;
    }
  }
  
  QListViewItem *newItem = new QListViewItem(item, newName);
  newItem->setRenameEnabled(0, true);
  queryList->setCurrentItem(newItem);
  newItem->startRename(0);
}
void CwinQueryListTab::deleteItem(){
  
  QListViewItem *item = queryList->currentItem();

  QString msgStr = "delete '" + item->text(0) + "' ?";
  
  if(KMessageBox::warningYesNo(this, msgStr) == 4)
    return;
  
  if(item->depth() == 0){
    if(!doc->queryMap.deleteNode(item->text(0))){
      KMessageBox::error(this, doc->queryMap.getError());
      return;
    }
  }else if(item->depth() <= 2){
    QListViewItem *parent = item->parent();
    if(item->depth() == 1){
      if(!doc->queryMap.deleteNode(parent->text(0), item->text(0))){
        KMessageBox::error(this, doc->queryMap.getError());
        return;
      }
    }else{
      if(!doc->queryMap.deleteNode(parent->parent()->text(0), parent->text(0), item->text(0))){
        KMessageBox::error(this, doc->queryMap.getError());
        return;
      }
    }
  }

  slotSyncList();
}
void CwinQueryListTab::updateQuery(QListViewItem *item, bool name){
  QueryInput oQi, nQi;
  
  QString queryStr = item->text(0);
  QString dbStr = item->parent()->text(0);
  QString conStr = item->parent()->parent()->text(0);

  oQi.setConStr(conStr);
  oQi.setDbStr(dbStr);
  oQi.setQueryName(currentItemStr);

  nQi.setQueryName(queryStr);
  nQi.setQuery(queryText->text());
  nQi.setQueryText(queryDiscr->text());
  
  if(!doc->queryMap.updateQueryInput(&oQi, &nQi, name))
    KMessageBox::error(this, doc->queryMap.getError());
  
  slotSyncList();
}
void CwinQueryListTab::slotConMenuActivated(int id){
  int index = conMenu->indexOf(id);
  if(index == 0){
    newItem();
  }else if(index ==1){
    queryList->currentItem()->startRename(0);
  }else if(index == 2){
    deleteItem();
  }
  
}
void CwinQueryListTab::slotDbMenuActivated(int id){
  int index = dbMenu->indexOf(id);
  QListViewItem *current = queryList->currentItem();
  if(index == 0){
    newItem();
  }else if(index == 1){

    QString oldDbStr = cQInput.getDbStr();
    QString oldConStr = cQInput.getConStr();
  
    cQInput.setDbStr(current->text(0));
    cQInput.setConStr(current->parent()->text(0));
    
    if(!doc->queryMap.insertQueryInput(&cQInput)){
      KMessageBox::error(this, doc->queryMap.getError());

      if(cut){
        cQInput.setDbStr(oldDbStr);
        cQInput.setConStr(oldConStr);

        if(!doc->queryMap.insertQueryInput(&cQInput)){
          KMessageBox::error(this, doc->queryMap.getError());
        }
        
      }
      
    }

    cItem = 0;
    slotSyncList();

  }else if(index ==2){
    current->startRename(0);
  }else if(index == 3){
    deleteItem();
  }
}                                                 
void CwinQueryListTab::slotQueryMenuActivated(int id){
  int index = queryMenu->indexOf(id);
  QListViewItem *current = queryList->currentItem();

  if(index == 2 || index == 3){

    cQInput.setDbStr(current->parent()->text(0));
    cQInput.setConStr(current->parent()->parent()->text(0));
    cQInput.setQueryName(current->text(0));
    cItem = current;
    
    if(!doc->queryMap.getQueryInput(&cQInput)){
      KMessageBox::error(this, doc->queryMap.getError());
      return;
    }

    cut = false;
    
    if(index == 3){
      
      if(!doc->queryMap.deleteNode(current->parent()->parent()->text(0), current->parent()->text(0), current->text(0))){
        KMessageBox::error(this, doc->queryMap.getError());
        return;
      }
      cut = true;
      slotSyncList();
    }

  }
  
  if(index == 0){
    QueryInput qi;

    qi.setQueryName(current->text(0));
    qi.setDbStr(current->parent()->text(0));
    qi.setConStr(current->parent()->parent()->text(0));
    qi.setType(QueryInput::Free);

    if(!doc->queryMap.getQueryInput(&qi))
      return;
      
    if(!doc->sendQuery(qi))
      KMessageBox::error(this, doc->getError());
  }else if(index == 5){
    current->startRename(0);
  }else if(index == 6){
    deleteItem();
  }
}
void CwinQueryListTab::slotNewCon(){
  QString newName = getNewName(0);
  if(!doc->queryMap.insertCon(newName)){
    KMessageBox::error(this, doc->queryMap.getError());
    return;
  }
  QListViewItem *newItem = new QListViewItem(queryList, newName);
  newItem->setRenameEnabled(0, true);
  queryList->setCurrentItem(newItem);
  currentItemStr = newName;
  newItem->startRename(0);
}
