/***************************************************************************
                          resulttab.cpp  -  description
                             -------------------
    begin                : Die Apr 29 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cwinresulttab.h"
#include <qlabel.h>
#include <qmemarray.h>
#include <qstringlist.h>
#include <klocale.h>
#include <qdatetime.h>

CwinResultTab::CwinResultTab(KSqlShellDoc *document, SqlClient *sqlClient, Result *result, const QString & label, QWidget *parent, const char *name) : CwinSqlOutTab(document, sqlClient, result, label, parent,name) {

  transResPix = iLoader.loadIcon("transponse", KIcon::MainToolbar);

  recordPix = iLoader.loadIcon("record", KIcon::Small);
  transpRecordPix = iLoader.loadIcon("transp_record", KIcon::Small);
  
  labelText->append(makeHeaderText());
  
  updPix =  iLoader.loadIcon("update", KIcon::MainToolbar);
  updButton = new QPushButton(QIconSet(updPix),"",page);
  connect(updButton, SIGNAL(clicked()), this, SLOT(slotUpdate()));
  QToolTip::add(updButton, i18n("reload"));
  hLayout->addStretch();
  hLayout->addWidget(updButton);
  
  transpose = false;
  transposeButton = new QPushButton(QIconSet(transResPix),"", page);
  QToolTip::add(transposeButton, i18n("transpose result"));
  hLayout->addWidget(transposeButton);
  connect(transposeButton, SIGNAL(clicked()), this, SLOT(slotTransposeResult()));
 
  table = new Table(page);
  table->setSelectionMode(QTable::NoSelection);
  layout->addWidget(table);
  transposable = true;
  showResult();
}
CwinResultTab::~CwinResultTab(){
}
QString CwinResultTab::makeHeaderText(){
  QString h = "";
  h = sqlCl->queryInput.getConStr() + "  <b>" + i18n("DATABASE") + ":</b> " + sqlCl->queryInput.getDbStr();
  h += "  <b>" + i18n("TIME") + ":</b> " + QTime::currentTime().toString();
  if(!sqlCl->queryInput.getQueryName().isEmpty())
    h += "  <b>" + i18n("QUERY NAME") + ":</b> " + sqlCl->queryInput.getQueryName();
  h += "  <b>" + i18n("QUERY") + ":</b> <pre>" +  res->getStmt() + "</pre>";
  if(!sqlCl->queryInput.getQueryText().isEmpty())
    h += "<br> <b>" + i18n("DESCRIPTION") + ":</b><br><br><pre>" +  sqlCl->queryInput.getQueryText() + "</pre>";  
  return h;
}
void CwinResultTab::showResult(){
  
  int tabRowNum = table->numRows();
  int tabColNum = table->numCols();
  QMemArray<int> rowArray(tabRowNum);
  QMemArray<int> colArray(tabColNum);

  int i = 0;
  for(; i < tabRowNum; i++)
    rowArray[i] = i;
  for(i = 0; i < tabColNum; i++)
    colArray[i] = i;

  table->removeRows(rowArray);
  table->removeColumns(colArray);
  
  res->setFirstRow();

  int colNum =  res->getColNum();
  int rowNum = res->getRowNum();

  doc->getConfig()->setGroup("General Options");
  int maxRows = doc->getConfig()->readNumEntry("maxRows");

  if(rowNum > maxRows){
    QString warn = QString(i18n("Warning: There are more than %1 rows. Please set a limit. \n")).arg(maxRows);
    labelText->setColor("red");
    labelText->insertAt(warn,0,0);
    labelText->setColor("black");
    rowNum = maxRows;
  }

  if(transpose){
    table->insertRows(0, colNum);
    table->insertColumns(0, rowNum);
  
    for(int i = 0; i < colNum; i++){
      table->verticalHeader()->setLabel(i,QString(res->getColName(i)));
      table->setRowReadOnly(i, false); 
    }

    for(int j = 0; j < rowNum; j++){
      res->nextRow();    
      for(int i = 0; i < colNum; i++){
        table->setText(i, j, QString(res->getValue(i)));
      }
      table->setColumnReadOnly(j, true);
    }
  }else{
    table->insertRows(0, rowNum);
    table->insertColumns(0, colNum);
    
    for(int i = 0; i < colNum; i++){
      table->horizontalHeader()->setLabel(i,QString(res->getColName(i)));
      table->setColumnReadOnly(i, false);
    }
    for(int j = 0; j < rowNum; j++){
      res->nextRow();
      for(int i = 0; i < colNum; i++){
        table->setText(j, i, QString(res->getValue(i)));
      }
     
     table->setRowReadOnly(j, true);
    }
  }
}

void CwinResultTab::slotTransposeResult(){
  if(!transposable)
    return;
    
  transpose = transpose ? false : true;
  
  transposeResult();
  
  showResult();
}
void CwinResultTab::refresh(SqlClient *sqlClient){
  QString error = "";
  refreshQuery = "";
  
  for(uint i = 0; i < sqlClient->resultList.count();i++){
   //qDebug("stmt: %s error: %s", sqlClient->resultList.at(i)->getStmt().latin1(), sqlClient->resultList.at(i)->getError().latin1());
    
    if(!sqlClient->resultList.at(i)->stmtOk()){
      QString err = sqlClient->resultList.at(i)->getError();
      if(!err.isEmpty()){
        refreshQuery = sqlClient->queryInput.getRefreshQuery();
        error = "<font color=\"red\"><b>" + i18n("ERROR") + ":</b>  " + err + "</font><br>";          
      }
    }else if(sqlClient->resultList.at(i)->isResult()){
      
      delTab();
      
      sqlCl = sqlClient;
      res = sqlClient->resultList.at(i);
      refr();
      transposable = true;
      labelText->clear();
      showResult();
      labelText->append(error + makeHeaderText());
      setEnabled(true);
      return;
    }
  }
  
  labelText->setText(error + makeHeaderText());
  setEnabled(true);
}
void CwinResultTab::slotUpdate(){
  QueryInput qi = tabQueryInput;
  
  qi.setRefreshQuery(res->getStmt());
  
  qi.setResultLabel(tabLabel);
  qi.setCheckInput(false);
  qi.setRefresh(true);

  //qDebug(">>%s", res->getStmt().latin1());

  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());
  else
    setEnabled(false);
}