/***************************************************************************
                          cwinselallrestab.cpp  -  description
                             -------------------
    begin                : Don Mai 8 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <klocale.h>
#include <qiconset.h>
#include "cwinselallrestab.h"
#include "queryinput.h"


CwinSelAllResTab::CwinSelAllResTab(KSqlShellDoc *document, SqlClient *sqlClient, Result * result, const QString & label, QWidget *parent, const char *name) : CwinResultTab(document, sqlClient, result, label, parent, name) {
  minRow = 0;
  maxRow = 0;
  currentPos = -1;
  menuAction = "";
  
  if(res->getRowNum() > 0){
    minRow = sqlCl->queryInput.getOffset() + 1;
    maxRow  = sqlCl->queryInput.getOffset() + res->getRowNum();
  }
  
  QStringList colList;
  colList.append(QString(""));
  for(int i = 0; i < res->getColNum(); i++)
    colList.append(res->getColName(i));

  doc->getConfig()->setGroup("General Options");
  int maxConfRows = doc->getConfig()->readNumEntry("maxRows");

  if(res->getRowNum() < maxConfRows)
    selWidget = new SelectWidget(sqlCl->queryInput.getOrderBy(), minRow, maxRow, &colList, page);
  else
    selWidget = new SelectWidget(sqlCl->queryInput.getOrderBy(), maxRow + 1, maxRow + maxConfRows, &colList, page);

  hLayout->insertWidget(0, selWidget);

  connect(selWidget, SIGNAL(signalNextRows(int, int, const QString &)), this, SLOT(slotNextRows(int, int, const QString &)));

  labelText->setText(makeHeaderText());
  
  undoPix =  iLoader.loadIcon("undoaction", KIcon::Small);
  
  menu = new QPopupMenu();
  menu->insertItem(i18n("update"), 0);
  menu->insertItem(i18n("insert"), 1);
  menu->insertItem(i18n("delete"), 2);
  menu->insertItem(QIconSet(undoPix), i18n("back"), 3);

  connect(table->verticalHeader(), SIGNAL(clicked(int)), this, SLOT(slotMenu(int)));
  connect(menu, SIGNAL(activated(int)), this, SLOT(slotAction(int)));

  undoPix =  iLoader.loadIcon("undoaction", KIcon::MainToolbar);
  undoButton = new QPushButton(QIconSet(undoPix),"",page);
  QToolTip::add(undoButton, i18n("back"));
  hLayout->insertWidget(2, undoButton);

  connect(undoButton, SIGNAL(clicked()), this, SLOT(slotUndo()));
}
CwinSelAllResTab::~CwinSelAllResTab(){
}
QString CwinSelAllResTab::makeHeaderText(){
  QString h = "";
  h = sqlCl->queryInput.getConStr() + "  <b>" + i18n("DATABASE") + ":</b> ";
  h += sqlCl->queryInput.getDbStr() + "  <b>" + i18n("TABLE") + ":</b> " + sqlCl->queryInput.getTabStr();
  h += "  <b>" + i18n("TIME") + ":</b> " + QTime::currentTime().toString() + "  ";
  h += "<b>" + i18n("ROWS") + ":</b> " + QString("%1 - %2").arg(minRow).arg(maxRow);
  QString stmt = "";
  if(refreshQuery.isEmpty()){
    stmt = res->getStmt();
  }else{
    stmt =  refreshQuery;
  }
  h += "  <b>" + i18n("QUERY") + ":</b> <pre>" + stmt + "</pre>";
  return h;
}
void CwinSelAllResTab::transposeResult(){
  if(transpose){
    disconnect(table->verticalHeader(), SIGNAL(clicked(int)), this, SLOT(slotMenu(int)));
    connect(table->horizontalHeader(), SIGNAL(clicked(int)), this, SLOT(slotMenu(int)));
  }else{
    disconnect(table->horizontalHeader(), SIGNAL(clicked(int)), this, SLOT(slotMenu(int)));
    connect(table->verticalHeader(), SIGNAL(clicked(int)), this, SLOT(slotMenu(int)));
  }
}
void CwinSelAllResTab::slotNextRows(int offset, int count, const QString & orderBy){
  QueryInput qi = sqlCl->queryInput;

  if(orderBy != qi.getOrderBy())
    qi.setOrderBy(orderBy);
    
  qi.setType(QueryInput::Select);
  qi.setResultLabel(tabLabel);
  qi.setLimit(offset, count);
  qi.setCheckInput(false);
  qi.setRefresh(false);

  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());
}
void CwinSelAllResTab::slotMenu(int pos){
  if(menuAction.isEmpty()){
    menu->setItemEnabled(menu->idAt(0), true);
    menu->setItemEnabled(menu->idAt(1), true);
    menu->setItemEnabled(menu->idAt(2), true);
    menu->setItemEnabled(menu->idAt(3), false);
    currentPos = pos;
  }else{
    menu->setItemEnabled(menu->idAt(0), false);
    menu->setItemEnabled(menu->idAt(1), false);
    menu->setItemEnabled(menu->idAt(2), false);
    menu->setItemEnabled(menu->idAt(3), true);
  }
  
  if(transpose){
    QRect rect = table->horizontalHeader()->sectionRect(pos);
    menu->popup( table->horizontalHeader()->mapToGlobal(rect.topLeft()) += QPoint(25, 8));
  }else{
    QRect rect = table->verticalHeader()->sectionRect(pos);
    menu->popup( table->verticalHeader()->mapToGlobal(rect.topLeft()) += QPoint(25, 8));
  }
}

QString CwinSelAllResTab::makeWhere(){
  int colNum = res->getColNum();
  QString where = " WHERE ";
  QString val;
  QStringList whereList;

  if(sqlTable.isOid()){
    where += "oid = " + oldRow[0];
  }else{
  
    for(int i = 0; i < colNum; i++){
      if(sqlTable.isPKey()){
        if(sqlTable.mapList["Key"][i].isEmpty())
          continue;
      }
      val = oldRow[i];
      if(val != "NULL")
        val = " = '" + val + "'";
      else
        val = " IS NULL";
      
      whereList.append(res->getColName(i) + " " + val);
    }

    where += whereList.join(" and ");
  }
  
  return where;
}
void CwinSelAllResTab::getRow(bool edit){
   if(transpose){

      for(int i = 0; i < table->numRows(); i++){
        oldRow.append(table->text(i, currentPos));
      }
      if(edit){
        table->setColumnReadOnly(currentPos, false);
        if(sqlTable.isOid())
          table->setRowReadOnly(0, true);
      }
    }else{

      for(int i = 0; i < table->numCols(); i++){
        oldRow.append(table->text(currentPos, i));
      }
      if(edit){
        table->setRowReadOnly(currentPos, false);
        if(sqlTable.isOid())
          table->setColumnReadOnly(0, true);
      }
    }
}
void CwinSelAllResTab::slotAction(int id){
  int index = menu->indexOf(id);
  transposable = false;
  
  sqlTable = doc->refMap->getSqlTable(sqlCl->queryInput.getConStr(), sqlCl->queryInput.getDbStr(), sqlCl->queryInput.getTabStr());
  
  if(index == 0){
    menuAction = "update";
    getRow(true);
   
  }else if(index == 1){
    menuAction = "insert";
    
    if(transpose){
      table->insertColumns(currentPos);
      table->setColumnReadOnly(currentPos, false);
      if(sqlTable.isOid())
        table->setRowReadOnly(0, true);
    }else{
      table->insertRows(currentPos);
      table->setRowReadOnly(currentPos, false);
      if(sqlTable.isOid())
        table->setColumnReadOnly(0, true);
    }

    for(int i = 0; i < res->getColNum(); i++){
      int row, col;
      if(transpose){
        row = i;
        col = currentPos;
      }else{
        row = currentPos;
        col = i;
      }
      if(sqlTable.isOid() && i > 0){
       // qDebug("row %d, col %d, i-1 %d, val %s", row, col ,i - 1,  sqlTable.mapList["Default"][i - 1].latin1());
        table->setText(row, col, sqlTable.mapList["Default"][i - 1]);
      }else if(!sqlTable.isOid())
        table->setText(row, col, sqlTable.mapList["Default"][i]);
    }

  }else if(index == 2){
    menuAction = "delete";
    getRow(false);
  }else if(index == 3){
    slotUndo();
  }

  if(index == 0 || index == 1 || index == 2){
    if(transpose){
      table->horizontalHeader()->setLabel(currentPos, QIconSet(transpRecordPix), i18n(menuAction));
    }else{
      table->verticalHeader()->setLabel(currentPos, QIconSet(recordPix), i18n(menuAction));
      table->setLeftMargin(80);
    }
  }
}

void CwinSelAllResTab::slotUpdate(){
  
  int colNum = res->getColNum();
  QString query;
  
  if(menuAction == "update"){
    if(!sqlTable.isOid() && !sqlTable.isPKey()){
      if(KMessageBox::warningYesNo(this, "if there is no primary key, all rows with old fields will be updated") == 4){
        slotUndo();
        return;
      }
    }

    query = "UPDATE " + sqlCl->queryInput.getTabStr() + " SET ";
    
  }else if(menuAction == "insert"){
    query = "INSERT INTO " + sqlCl->queryInput.getTabStr() + " VALUES (";
  }else if(menuAction == "delete"){
    if(!sqlTable.isOid() && !sqlTable.isPKey()){
      if(KMessageBox::warningYesNo(this, "if there is no primary key, all rows with old fields will be deleted") == 4){
        slotUndo();
        return;
      }
    }
  }         
    
  QStringList qList;

  table->editEnd();
 
  for(int i = 0; i < colNum; i++){
    if(i == 0 && sqlTable.isOid())
      continue;
    
    int row, col;
    if(transpose){
      row = i;
      col = currentPos;
    }else{
      row = currentPos;
      col = i;
    }

    
    QString val = table->text(row, col);

  
    if(val != "NULL" && val.at(0) != '\''){
      val = "'" + val + "'";
    }
  
  
    if(menuAction == "update"){
      qList.append(res->getColName(i) + " = " + val);
    }else if(menuAction == "insert"){
      qList.append(val);
    }   
  }
  
  if(menuAction == "update"){
    query += qList.join(", ") + makeWhere();
  }else if(menuAction == "insert"){
    query += qList.join(",") + ")";
  }else if(menuAction == "delete"){
    query = "DELETE FROM " + sqlCl->queryInput.getTabStr() + makeWhere();
  }

  QueryInput qi = tabQueryInput;

  if(menuAction.isEmpty()){
    qi.setRefreshQuery(qi.getRealQuery());
  }else{ 
    qi.setRefreshQuery(query + ";" + qi.getRealQuery());
  }
  
  qi.setResultLabel(tabLabel);
  qi.setCheckInput(false);
  qi.setRefresh(true);
  
  //qDebug(">>%s", qi.getQuery().latin1());
  
  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());
  else
    setEnabled(false);
}

void CwinSelAllResTab::slotUndo(){
  showResult();
  oldRow.clear();
  menuAction = "";
  transposable = true;
}
void CwinSelAllResTab::refr(){
  oldRow.clear();
  menuAction = "";
}