/***************************************************************************
                          cwintab.cpp  -  description
                             -------------------
    begin                : Mit Apr 30 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qpoint.h>
#include <klocale.h>
#include <qiconset.h>
#include <qvaluelist.h>
#include <qregexp.h>
#include "cwintab.h"
#include "centralwin.h"

CwinTab::CwinTab(const QString & label, QWidget *parent, const char *name ) : QWidget(parent,name) {
  
  delTabPix =  iLoader.loadIcon("del_tab", KIcon::MainToolbar);
  plugTabPix = iLoader.loadIcon("plug", KIcon::MainToolbar);
  unplugTabPix = iLoader.loadIcon("unplug", KIcon::MainToolbar);
  
  tabBar = (CentralWin*)parent;
  tabLabel = label;
  plug = true;
  
  mLayout = new QGridLayout(this);
  splitter = new QSplitter(Vertical,this);

  headerPage = new QWidget(splitter);
  hHeaderLayout = new QHBoxLayout(headerPage);
  
  
  labelText = new QTextEdit(headerPage);
  labelText->setReadOnly(true);
  labelText->setMinimumHeight(20);
  setLabelTextHeight(4);
  hHeaderLayout->addWidget(labelText);

  vHeaderLayout = new QVBoxLayout(hHeaderLayout);
  
  bGrid = new QGrid(2, headerPage);

  vHeaderLayout->addWidget(bGrid);
  vHeaderLayout->addStretch();
  
  plugButton = new QPushButton(QIconSet(unplugTabPix),"", bGrid);
  QToolTip::add(plugButton, i18n("unplug"));
  delButton = new QPushButton(QIconSet(delTabPix),"", bGrid);
  QToolTip::add(delButton, i18n("close"));
  

  page = new QWidget(splitter);
  layout = new QVBoxLayout(page);
  hLayout = new QHBoxLayout(layout);
  
  mLayout->addWidget(splitter,0,0);
  
  connect(plugButton, SIGNAL(clicked()), this, SLOT(slotPlugTab()));
  connect(delButton, SIGNAL(clicked()), this, SLOT(slotDelTab()));
  
}
CwinTab::~CwinTab(){
}

void CwinTab::slotPlugTab(){
  if(plug){
    tabBar->removePage(this);
    reparent(0,  QPoint(100, 100));
    plugButton->setIconSet(QIconSet(plugTabPix));
    QToolTip::add(plugButton, i18n("plug"));
    setCaption(tabLabel);
    show();
    plug = false;
  }else{
    reparent(tabBar, QPoint(0,0));
    tabBar->addTab(this,tabLabel);
    plugButton->setIconSet(QIconSet(unplugTabPix));
    QToolTip::add(plugButton, i18n("unplug"));
    tabBar->showPage(this);
    plug = true;
  } 
}
void CwinTab::setLabelTextHeight(int height){
  QValueList<int> splitterSize = splitter->sizes();
  splitterSize[0] = height;
  splitterSize[1] = 100;
  splitter->setSizes(splitterSize);
}
void CwinTab::slotDelTab(){
  delTab();
  tabBar->labelMap.remove(tabLabel);
  delete this;
}
void CwinTab::closeEvent(QCloseEvent *ce){
  slotDelTab();
}
