/***************************************************************************
                          cwintabletab.cpp  -  description
                             -------------------
    begin                : Don Jun 19 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cwintabletab.h"
#include <klocale.h>
#include <qiconset.h>
#include <qtabwidget.h>
 #include <qmemarray.h>
 #include <qdatetime.h>
 #include <kmessagebox.h>
 
CwinTableTab::CwinTableTab(KSqlShellDoc *document,SqlTable & sqlTable, const QString & label, QWidget *parent, const char *name ) : CwinTab(label, parent,name) {
  tab = sqlTable;
  table = 0;
  doc = document;

  
  updPix =  iLoader.loadIcon("update", KIcon::MainToolbar);
  updButton = new QPushButton(QIconSet(updPix),"",page);
  QToolTip::add(updButton, i18n("reload"));
  hLayout->addStretch();
  hLayout->addWidget(updButton);

  connect(updButton, SIGNAL(clicked()), this, SLOT(slotRefresh()));
  
  rows = tab.getFieldNum();
  cols = tab.attrList.count();
  table = new QTable(page);
  layout->addWidget(table);
  
  showColumns();
}
CwinTableTab::~CwinTableTab(){
}
QString CwinTableTab::makeHeaderText(){
  QString h = "";
  h = tab.getConStr() + "  <b>" + i18n("DATABASE") + ":</b> ";
  h += tab.getDbStr() + "  <b>" + i18n("TABLE") + ":</b> " + tab.getTabStr();
  h += "  <b>" + i18n("TIME") + ":</b> " + QTime::currentTime().toString() + "  ";
  return h;
}
void CwinTableTab::delTab(){
}
void CwinTableTab::showColumns(){
  labelText->setText(makeHeaderText());
  table->insertRows(0, rows);
  table->insertColumns(0, cols);
  for(int c = 0; c < cols; c++){
    QString colStr = tab.attrList[c];
    table->horizontalHeader()->setLabel(c,i18n(colStr));
    for(int r = 0; r < rows; r++){
      table->setText(r, c, tab.mapList[colStr][r]);
    }
  }
  
}
void CwinTableTab::slotRefresh(){
  if(!doc->clQueueMap[tab.getConStr()]->isConnect()){
    KMessageBox::error(this, i18n("is not connected!"));
    return;
  }
  
  QMemArray<int> rowArray(tab.getFieldNum());
  QMemArray<int> colArray(tab.attrList.count());

  int i= 0;
  for(; i < tab.getFieldNum(); i++)
    rowArray[i] = i;
  for(i = 0; i < tab.attrList.count(); i++)
    colArray[i] = i;

  table->removeRows(rowArray);
  table->removeColumns(colArray);
  
  //qDebug(">%s< >%s< >%s<", tab.getConStr().latin1(), tab.getDbStr().latin1(), tab.getTabStr().latin1());
  tab = doc->refMap->getSqlTable(tab.getConStr(), tab.getDbStr(), tab.getTabStr());
  rows = tab.getFieldNum();
  cols = tab.attrList.count();
  showColumns();
}