/***************************************************************************
                          fielddlg.cpp  -  description
                             -------------------
    begin                : Mit Jul 16 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <klocale.h>
#include <kmessagebox.h>
#include <qstringlist.h>
#include "fielddlg.h"

FieldDlg::FieldDlg(KSqlShellDoc *document, SqlTable & sqlTab, QWidget *parent, const char *name) 
                            : KDialogBase(parent,name, true, name, Ok | Cancel, Ok, true) {

  doc = document;
  sqlTable = sqlTab;
  action = sqlTable.getTableAction();
  page = new QWidget(this);
  setMainWidget(page);
  vLayout = new QVBoxLayout(page);
  newFieldIdx = -1;
  QString header = "";
  if(action == SqlTable::AddField)
    header = "add field to '" + sqlTable.getConStr() + "/" + sqlTable.getDbStr() + "'  '" + sqlTable.getTabStr() +"'";
  else
    header = "change field: '" + sqlTable.getConStr() + "/" + sqlTable.getDbStr() + "/" + sqlTable.getTabStr() + "'  '" + sqlTable.getChangeField() + "'";

  dbType = doc->clQueueMap[sqlTable.getConStr()]->getType();
  sqlTypes = doc->clQueueMap[sqlTable.getConStr()]->getSqlTypes();
  
  vGroupBox = new QVGroupBox(header, page);
  table = new Table(1, 6, vGroupBox);
  table->setMaximumHeight(table->rowHeight(0) * 3 + 5);
  table->verticalHeader()->setLabel(0,"");

  table->horizontalHeader()->setLabel(0, i18n(i18n("Name")));
  table->horizontalHeader()->setLabel(1,i18n(i18n("Type")));
  table->horizontalHeader()->setLabel(2,i18n("Length, Dec"));
  table->horizontalHeader()->setLabel(3, i18n("Attribute"));
  table->horizontalHeader()->setLabel(4,i18n("Default Value"));
  table->horizontalHeader()->setLabel(5,i18n("Not Null"));

  if(dbType == ClQueue::MYSQL){
    table->insertColumns(6);
    table->horizontalHeader()->setLabel(6, i18n("Values"));
    table->setItem(0,6, new QTableItem(table, QTableItem::OnTyping, ""));
  }

  table->setSelectionMode(QTable::NoSelection);

  if(action == SqlTable::AddField)
    table->setItem(0, 0, new QTableItem(table, QTableItem::OnTyping, ""));
  else
    table->setItem(0, 0, new QTableItem(table, QTableItem::Never, ""));

  table->setItem(0, 1, new QComboTableItem(table, sqlTypes.typeList, true));
  table->setItem(0, 2, new QTableItem(table, QTableItem::OnTyping, ""));
  table->setItem(0, 3, new QComboTableItem(table,  0, true));
  table->setItem(0, 4, new QTableItem(table, QTableItem::OnTyping, ""));
  table->setItem(0, 5, new QCheckTableItem(table,""));


  // for "change field" insert values
  if(action == SqlTable::ChangeField){
    ((QTableItem*)table->item(0,0))->setText(sqlTable.getChangeField());

    int fIdx = sqlTable.getFieldIndex(sqlTable.getChangeField());

    QStringList typeList = splitType(sqlTable.mapList["Type"][fIdx]);
    QString typeName = typeList[0];

    ((QComboTableItem*)table->item(0, 1))->setCurrentItem(typeName);

    slotValueChanged(0, 1);

    if(typeList.count() >= 2){
      if(dbType == ClQueue::MYSQL && sqlTypes.sqlTypeMap[typeName].selfDef){
        ((QTableItem*)table->item(0,6))->setText(typeList[1]);
      }else{
        ((QTableItem*)table->item(0,2))->setText(typeList[1]);
      }
    }
    if(typeList.count() > 2){
      QString attr = typeList[2].stripWhiteSpace();
      ((QComboTableItem*)table->item(0, 3))->setCurrentItem(attr);
    }

    ((QTableItem*)table->item(0,4))->setText(sqlTable.mapList["Default"][fIdx]);

    if(sqlTable.mapList["NULL"][fIdx] != "YES")
      ((QCheckTableItem *)table->item(0, 5))->setChecked(true);

  }else if(dbType == ClQueue::MYSQL){
    table->insertColumns(7);
    table->horizontalHeader()->setLabel(7, i18n("Position"));
    QStringList posList;
    posList.append("");
    posList.append("FIRST");
    for(int i = 0; i < sqlTable.getFieldNum(); i++){
      QString lStr = "AFTER " + sqlTable.mapList["Name"][i];
      posList.append(lStr);
    }
    table->setItem(0,7, new QComboTableItem(table, posList, false));
  }

  vLayout->addWidget(vGroupBox);

  connect(table, SIGNAL(valueChanged(int , int )), this, SLOT(slotValueChanged(int,int)));
}
FieldDlg::~FieldDlg(){
}
void FieldDlg::slotValueChanged(int row, int col){

  if(col == 1){
    QComboTableItem *typeBox = (QComboTableItem*)table->item(row,col);
    QString type = typeBox->currentText();

    ((QComboTableItem*)table->item(row, 3))->setStringList(sqlTypes.opt0Map[type]);

    table->item(row, 2)->setEnabled(sqlTypes.sqlTypeMap[type].lengDec);
    table->setText(row, 2, "");
    table->setText(row, 4, "");
    ((QCheckTableItem *)table->item(0, 5))->setChecked(false);

    if(dbType == ClQueue::MYSQL){
      table->item(row, 6)->setEnabled(sqlTypes.sqlTypeMap[type].selfDef);
      table->setText(row, 6, "");
    }

  }
}
QStringList FieldDlg::splitType(const QString & typeStr){
  QStringList list;
  char * pT = (char *)typeStr.latin1();
  bool t = true;
  bool tt = true;

  QString s = "";

  while(*pT){

    if(*(pT - 1) != '\\' || *(pT - 2) == '\\'){
      if(*pT == '\'')
        t = !t;
      if(*pT == '\"')
        tt = !tt;
    }

    if((*pT == '(' || *pT == ')') && t && tt){
      list.append(s);
      s = "";
    }else
      s += *pT;

    pT++;
  }

  if(!s.isEmpty())
    list.append(s);

  return list;
}
void FieldDlg::slotOk(){

  table->editEnd();
  
  if(newFieldIdx == -1)
    newFieldIdx = sqlTable.getFieldNum();

  sqlTable.setChangeField(table->item(0,0)->text());

  sqlTable.mapList["Name"][newFieldIdx] = table->item(0,0)->text();

  QString type = table->item(0, 1)->text();

  if(!table->item(0,2)->text().isEmpty()){
    type += "(" + table->item(0,2)->text() + ")";
  }else if(sqlTypes.sqlTypeMap[type].selfDef){
    type += "(" + table->item(0,6)->text() + ")";
  }

  type += " " + table->item(0,3)->text();

  sqlTable.mapList["Type"][newFieldIdx] = type;
  sqlTable.mapList["Default"][newFieldIdx] = table->item(0,4)->text();

  if(((QCheckTableItem *)table->item(0,5))->isChecked())
    sqlTable.mapList["NULL"][newFieldIdx] = "";
  else
    sqlTable.mapList["NULL"][newFieldIdx] = "YES";
    
  if(dbType == ClQueue::MYSQL && action == SqlTable::AddField){
    sqlTable.setNewFieldPos(table->item(0,7)->text());
  }

  if(doc->makeTable(sqlTable))
    delayedDestruct();
  else
    KMessageBox::error(this, doc->getError());
}
