/***************************************************************************
                          ksqlshell.cpp  -  description
                             -------------------
    begin                : Mit Feb 26 16:50:35 CET 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qstringlist.h>
// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kanimwidget.h>
// application specific includes
#include "ksqlshell.h"
#include "ksqlshellview.h"
#include "ksqlshelldoc.h"

#include "newcondlg.h"
#include "newdbdlg.h"
#include "pgnewdbdlg.h"
#include "mytabdlg.h"
#include "pgtabdlg.h"
#include "queryinput.h"
#include "savequerydlg.h"
#include "setupdlg.h"
#include "fielddlg.h"


#define ID_STATUS_MSG 1

KSqlShellApp::KSqlShellApp(QWidget* , const char* name):KMainWindow(0L, name)
{
  config=kapp->config();
   
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();

  readOptions();

  initNaviListActions();
  
  oldItemName = "";
  connect(view->naviList, SIGNAL(itemRenamed(QListViewItem *, int, const QString &)), this, SLOT(slotRenameListItem(QListViewItem *, int, const QString &)));
  connect(doc, SIGNAL(signalRunSync(bool)), this, SLOT(slotRunMovie(bool)));
  slotSetActionStatus();
  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  //fileSave->setEnabled(false);
  //fileSaveAs->setEnabled(false);
  //filePrint->setEnabled(false);
  //editCut->setEnabled(false);
  //editCopy->setEnabled(false);
  //editPaste->setEnabled(false);

}

KSqlShellApp::~KSqlShellApp()
{

}

void KSqlShellApp::initActions()
{
  //fileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  //fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  //fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  //fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  //fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  //fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  //fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  //filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  //fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileClose()), actionCollection());
  //editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  //editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  //editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

  //fileNewWindow->setStatusText(i18n("Opens a new application window"));
  //fileNew->setStatusText(i18n("Creates a new document"));
  //fileOpen->setStatusText(i18n("Opens an existing document"));
  //fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  //fileSave->setStatusText(i18n("Saves the actual document"));
  //fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  //fileClose->setStatusText(i18n("Closes the actual document"));
  //filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  //editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  //editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  //editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

  setup = new KAction(i18n("setup"), "configure", 0, this, SLOT(slotSetup()), actionCollection(), "");
    
  newConnect = new KAction(i18n("New Connection"), "connect_creating", 0, this, SLOT(slotNewConnect()), actionCollection(), "");
  saveQuery = new KAction(i18n("save query"), "filesave", 0, this, SLOT(slotSaveQuery()), actionCollection(), "");
  editQuerys = new KAction(i18n("query list"), 0, 0, this, SLOT(slotEditQueryList()), actionCollection(), "");
  refresh = new KAction(i18n("Refresh"), "reload", 0, this, SLOT(slotRefresh()), actionCollection(), "");
  
  conCon = new KAction(i18n("connect"), "connect_established", 0, this, SLOT(slotListConCon()), actionCollection(), "");
  conDiscon = new KAction(i18n("disconnect"), "connect_no", 0, this, SLOT(slotListConDiscon()), actionCollection(), "");
  conDel =  new KAction(i18n("delete connection"), "cancel", 0, this, SLOT(slotListConDel()), actionCollection(), "");
  conNewDb =  new KAction(i18n("new database"), "database", 0, this, SLOT(slotListConNewDb()), actionCollection(), "");
  conInfo = new KAction(i18n("info"), "info", 0, this, SLOT(slotListConInfo()), actionCollection(), "");
  
  dbNewTab = new KAction(i18n("new table"), "table", 0, this, SLOT(slotListDbNewTab()), actionCollection(), "");
  dbDel = new KAction(i18n("delete database"), "del_database", 0, this, SLOT(slotListDbDel()), actionCollection(), "");

  tabSelect =  new KAction(i18n("select"), "select", 0, this, SLOT(slotListTabSelect()), actionCollection(), "");
  tabSelectAll = new KAction(i18n("select all"), "select_all", 0, this, SLOT(slotListTabSelectAll()), actionCollection(), "");
  tabInsert = new KAction(i18n("insert"), "insert", 0, this, SLOT(slotListTabInsert()), actionCollection(), "");
  tabShowColumns = new KAction(i18n("show columns"), "showcolumns", 0, this, SLOT(slotListTabShowColumns()), actionCollection(), "");
  //tabChange = new KAction(i18n("change table"), 0, 0, this, SLOT(slotListTabChange()), actionCollection(), "");
  tabDel = new KAction(i18n("delete table"), "del_table", 0, this, SLOT(slotListTabDel()), actionCollection(), "");
  tabNewField = new KAction(i18n("new field"), "addfield", 0, this, SLOT(slotListTabNewField()), actionCollection(), "");
  tabRename = new KAction(i18n("rename table"), 0, 0, this, SLOT(slotStartRenameListItem()),actionCollection(), "");
  
  fieldChange  =  new KAction(i18n("change field"), "changefield", 0, this, SLOT(slotListFieldChange()), actionCollection(), "");
  fieldDel =  new KAction(i18n("delete field"), "delfield", 0, this, SLOT(slotListFieldDel()), actionCollection(), "");
  fieldRename = new KAction(i18n("rename field"), 0, 0, this, SLOT(slotStartRenameListItem()), actionCollection(), "");
  
  // use the absolute path to your ksqlshellui.rc file for testing purpose in createGUI();

  createGUI();
 
  conMenu = new QPopupMenu();
  queryMenu = new QPopupMenu();
  queryListMenu = new QPopupMenu();
  setupMenu = new QPopupMenu();
  viewMenu = new QPopupMenu();

  connect(queryListMenu, SIGNAL(activated(int)), this, SLOT(slotSendQuery(int)));
  menuBar()->insertItem(i18n("Connections"), conMenu,1,1);
  menuBar()->insertItem(i18n("View"), viewMenu, 1,2);
  menuBar()->insertItem(i18n("Queries"), queryMenu, 1,3);
  menuBar()->insertItem(i18n("Setup"), setupMenu, 1, 4);

  menuBar()->removeItem(menuBar()->idAt(5));
  
  newConnect->plug(conMenu);
  setup->plug(setupMenu);
  newConnect->plug(toolBar());

  saveQuery->plug(queryMenu);
  editQuerys->plug(queryMenu);
  refresh->plug(viewMenu);

  listConMenu = new QPopupMenu();
  listDbMenu = new QPopupMenu();
  listTabMenu = new QPopupMenu();
  listFieldMenu = new QPopupMenu();
  
  conDel->plug(toolBar());
  conCon->plug(toolBar());
  conDiscon->plug(toolBar());
  conInfo->plug(toolBar());
  refresh->plug(toolBar());
  
  conNewDb->plug(toolBar());
  dbDel->plug(toolBar());
  
  dbNewTab->plug(toolBar());
  
  //tabChange->plug(toolBar());
  //tabRename->plug(toolBar());
  tabDel->plug(toolBar());
  tabSelect->plug(toolBar());
  tabSelectAll->plug(toolBar());
  tabInsert->plug(toolBar());
  tabShowColumns->plug(toolBar());
  tabNewField->plug(toolBar());
  
  //fieldRename->plug(toolBar());
  fieldChange->plug(toolBar());
  fieldDel->plug(toolBar());
  
  saveQuery->plug(toolBar());

  movieTbId = KAction::getToolButtonID();
  toolBar()->insertAnimatedWidget(movieTbId, this, SLOT(slotMovieClicked()), QString("kde"));
  toolBar()->alignItemRight(movieTbId, true);
}
void KSqlShellApp::initNaviListActions(){
  
  connect(view->naviList,SIGNAL(contextMenuRequested(QListViewItem*, const QPoint &, int)), this, SLOT(slotListPopupMenu(QListViewItem*, const QPoint &, int)));
  connect(view->naviList,SIGNAL(selectionChanged()), this, SLOT(slotSetActionStatus()));

  conCon->plug(listConMenu);
  conDiscon->plug(listConMenu);
  conNewDb->plug(listConMenu);
  conInfo->plug(listConMenu);
  conDel->plug(listConMenu);
  

  dbNewTab->plug(listDbMenu);  
  listDbMenu->insertItem(i18n("run query"), queryListMenu);
  dbDel->plug(listDbMenu);

  tabSelect->plug(listTabMenu);
  tabSelectAll->plug(listTabMenu);
  tabInsert->plug(listTabMenu);
  tabShowColumns->plug(listTabMenu);
  //tabChange->plug(listTabMenu);
  tabRename->plug(listTabMenu);
  tabNewField->plug(listTabMenu);
  tabDel->plug(listTabMenu);

  fieldRename->plug(listFieldMenu);
  fieldChange->plug(listFieldMenu);
  fieldDel->plug(listFieldMenu);

  //init action state
 
}
void KSqlShellApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KSqlShellApp::initDocument()
{
  doc = new KSqlShellDoc(this);
  doc->newDocument();
}

void KSqlShellApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.
  view = new KSqlShellView(this);
  doc->addView(view);
  setCentralWidget(view);	
  setCaption(doc->URL().fileName(),false);

}

void KSqlShellApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  //fileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
}


KSqlShellDoc *KSqlShellApp::getDocument() const
{
  return doc;
}

void KSqlShellApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  config->writeEntry("Refresh", doc->getSyncNaviListTime());

  //reading Connections
  config->deleteGroup("Connections");
  config->setGroup("Connections");

  for(doc->mapIt = doc->clQueueMap.begin(); doc->mapIt != doc->clQueueMap.end(); ++doc->mapIt){
    
    QStringList vList;
    vList.append(doc->mapIt.data()->getTypeStr());
    vList.append(doc->mapIt.data()->getHost());
    vList.append(doc->mapIt.data()->getPort());
    vList.append(doc->mapIt.data()->getUser());
    vList.append(doc->mapIt.data()->getPw());
    vList.append(doc->mapIt.data()->getDefDb());
    
    config->writeEntry(doc->mapIt.key(), vList);
  }
  
  //fileOpenRecent->saveEntries(config,"Recent Files");
}

void KSqlShellApp::slotRunMovie(bool run){
  if(run)
    toolBar()->animatedWidget(movieTbId)->start();
  else
    toolBar()->animatedWidget(movieTbId)->stop();
}

void KSqlShellApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
  //fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }

   QMap<QString, QString> confMap = config->entryMap("Connections");
   QMap<QString, QString>::iterator confIt = confMap.begin();
   for(; confIt != confMap.end(); ++confIt){
    QStringList entryList = QStringList::split(',', confIt.data(), true);
    doc->addConnect(ClQueue::getDbType(entryList[0]), entryList[1], entryList[2].toInt(), entryList[3], entryList[4], entryList[5]); 
   }
   
  if(config->readNumEntry("maxRows") == 0)
    config->writeEntry("maxRows", 2000);
    
  doc->setSyncNaviListTime(config->readNumEntry("Refresh"));
  QString file = config->readPathEntry("QueryFile");
  if(file.isEmpty()){
    file = "ksqlshellquery.xml";
    config->writePathEntry("QueryFile", file);
  }
  
  doc->queryMap.setFile(config->readPathEntry("QueryFile"));
  
    if(!doc->queryMap.loadFile())
      KMessageBox::sorry(this, doc->queryMap.getError());

}

void KSqlShellApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}


void KSqlShellApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);
  	
    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}		

bool KSqlShellApp::queryClose()
{
  return doc->saveModified();
}

bool KSqlShellApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KSqlShellApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  KSqlShellApp *new_window= new KSqlShellApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    doc->newDocument();		
    setCaption(doc->URL().fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*|All files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(), false);
      //fileOpenRecent->addURL( url );
    }
  }
  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    doc->openDocument(url);
    setCaption(url.fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	
  doc->saveDocument(doc->URL());

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    doc->saveDocument(url);
    //fileOpenRecent->addURL(url);
    setCaption(url.fileName(),doc->isModified());
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
}

void KSqlShellApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KSqlShellApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void KSqlShellApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KSqlShellApp::slotNewConnect()
{
  slotStatusMsg(i18n("create new connection ..."));
  NewConDlg ncDlg(this, i18n("Connection"), doc);
  ncDlg.exec();
  slotStatusMsg(i18n("Ready."));
}
void KSqlShellApp::slotSendQuery(int id){
  QString conStr =  view->naviList->currentItem()->parent()->text(0);
  QString db = view->naviList->currentItem()->text(0);
  
  QString err = "";
  err = doc->refMap->getError(conStr, db);
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }
  
  QueryInput qi;
  qi.setQueryName(queryListMenu->text(id)); 
  qi.setConStr(conStr);
  qi.setDbStr(db);
  qi.setType(QueryInput::Free);

  if(!doc->queryMap.getQueryInput(&qi))
    return;
      
  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());

}
void KSqlShellApp::slotSaveQuery(){
  slotStatusMsg(i18n("save query ..."));
  
  QueryInput qi;        
  view->shell->getCurrentQueryInput(&qi);
  SaveQueryDlg sqDlg(this, i18n("Query"), doc, &qi);
  sqDlg.exec();

  if(view->cwin->queryList)
    view->cwin->qListTab->slotSyncList();
    
  slotStatusMsg(i18n("Ready."));
}
void KSqlShellApp::slotEditQueryList(){
  view->cwin->showQueryList(doc);
}
void KSqlShellApp::initQueryMenu(QListViewItem *item){
  QString db = item->text(0);
  QString con = item->parent()->text(0);
  QStringList qList = doc->queryMap.getQueryNameList(con, db);
  
  if(qList.isEmpty()){
    listDbMenu->setItemEnabled(listDbMenu->idAt(1), false);
    return;
  }
  
  listDbMenu->setItemEnabled(listDbMenu->idAt(1), true);
  queryListMenu->clear();
  
  QStringList::Iterator it = qList.begin();
  for(; it != qList.end(); ++it)
    queryListMenu->insertItem(*it);
    
}
void KSqlShellApp::slotListPopupMenu(QListViewItem* listItem, const QPoint &p, int con){
  
  if(!listItem)          // QListView send Signal contextMenuRequested without valid QListViewItem pointer ???
    return;

  if (listItem->depth() == 0)
    listConMenu->popup(p);
  else if (listItem->depth() == 1){
    initQueryMenu(listItem);
    listDbMenu->popup(p);
  }else if(listItem->depth() == 2)
    listTabMenu->popup(p);
  else
    listFieldMenu->popup(p);

}

void KSqlShellApp::slotListConCon(){
  if(doc->connect(view->naviList->currentItem()->text(0))){
    slotSetActionStatus();
  }else
    KMessageBox::error(this, doc->getError());
}
void KSqlShellApp::slotListConDiscon(){
  QString conStr = view->naviList->currentItem()->text(0);
  if(doc->clQueueMap[conStr]->isRunning())
    if(KMessageBox::warningYesNo(this, i18n("You have running querys, disconnect ?")) == 4)
      return;
      
  doc->disconnect(conStr);

  slotSetActionStatus();
}
void KSqlShellApp::slotListConDel(){
  doc->delConnect(view->naviList->currentItem()->text(0));
  slotSetActionStatus();
}
void KSqlShellApp::slotListConInfo(){
  QString con = view->naviList->currentItem()->text(0);
  view->cwin->showInfoTab(doc, con);
}
void KSqlShellApp::slotListConNewDb(){
  slotStatusMsg(i18n("create new database ..."));
  QString conStr = view->naviList->currentItem()->text(0);
  
  QString err = "";
  err = doc->refMap->getError(conStr);
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }
  
  if(doc->clQueueMap[conStr]->getType() == ClQueue::MYSQL){
    NewDbDlg ndbDlg(this, i18n("New Database"), doc, conStr);
    ndbDlg.exec();
  }else{
    PgNewDbDlg ndbDlg(this, i18n("New Database"), doc, conStr);
    ndbDlg.exec();
  }  
   
   slotStatusMsg(i18n("Ready"));
}                
void KSqlShellApp::slotListDbNewTab(){
  slotStatusMsg(i18n("create new table ..."));
  
  QString conStr = view->naviList->currentItem()->parent()->text(0);
  QString db =  view->naviList->currentItem()->text(0);

  QString err = "";
  err = doc->refMap->getError(conStr, db);
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;  
  }
  
  if(doc->clQueueMap[conStr]->getType() == ClQueue::MYSQL){
    MyTabDlg tabDlg(this, i18n("New Table"), doc, conStr, db);
    tabDlg.startDlg();
  }else{
    PgTabDlg tabDlg(this, i18n("New Table"), doc, conStr, db);
    tabDlg.startDlg();
  }   
    
  slotStatusMsg(i18n("Ready"));
}
void KSqlShellApp::slotListDbDel(){
  QString db = view->naviList->currentItem()->text(0);
  QString conStr = view->naviList->currentItem()->parent()->text(0);
  
  QString err = "";
  err = doc->refMap->getError(conStr, db);
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }
  
  QString msgStr = i18n("drop database ") + db;
  if(KMessageBox::warningYesNo(this, msgStr) == 4)
    return;
  if(!doc->delDb(conStr, db))
     KMessageBox::error(this, doc->getError());
  slotSetActionStatus();
}
void KSqlShellApp::slotListTabSelect(){
  QString tab = view->naviList->currentItem()->text(0);
  QString db =  view->naviList->currentItem()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->text(0);

  SqlTable sqlTable = doc->refMap->getSqlTable(con, db, tab);
  if(sqlTable.getTabStr().isEmpty()){
    KMessageBox::error(this, i18n("table not found"));
    return;  
  }
  QString err = sqlTable.getError();
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }                                                 
  
  QString query = "select ";
  
  if(sqlTable.isOid())
    query += "oid, ";
  
  query += "* from " + tab + " ";
  
  int fieldNum = sqlTable.getFieldNum();

  QueryInput qi;
  
  QStringList valList;

  if(sqlTable.isOid())
    valList.append(" <input \" oid \"></input>");
  
  for(int i = 0; i < fieldNum; i++){
    QString name = sqlTable.mapList["Name"][i];
    QString val = " <input \"" + name + "\" \"" + sqlTable.mapList["Type"][i] + "\"></input>";
    valList.append(val);
  }

  query += valList.join(" ");
    
  qi.setQueryName(i18n("Select"));
  qi.setTabStr(tab);
  qi.setDbStr(db);
  qi.setConStr(con);
  qi.setQuery(query);
  qi.setShowQuery(false);
  config->setGroup("General Options");
  qi.setLimit(0, config->readNumEntry("maxRows"));
  qi.setType(QueryInput::Select);
  
  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());
  
}
void KSqlShellApp::slotListTabSelectAll(){
  QString tab = view->naviList->currentItem()->text(0);
  QString db =  view->naviList->currentItem()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->text(0);
  
  SqlTable sqlTable = doc->refMap->getSqlTable(con, db, tab);
  if(sqlTable.getTabStr().isEmpty()){
    KMessageBox::error(this, i18n("table not found"));
    return;
  }
  QString err = sqlTable.getError();
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }
  
  QueryInput qi;
  
  qi.setTabStr(tab);
  qi.setDbStr(db);
  qi.setConStr(con);
  config->setGroup("General Options");
  qi.setLimit(0, config->readNumEntry("maxRows"));
  qi.setType(QueryInput::Select);

  QString query = "select ";
  
  if(sqlTable.isOid())
    query += "oid, ";
    
  query += "* from " + tab + " ";
  
  qi.setQuery(query);
  
  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());
  
}
void KSqlShellApp::slotListTabInsert(){
  QString tab = view->naviList->currentItem()->text(0);
  QString db =  view->naviList->currentItem()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->text(0);

  SqlTable sqlTable = doc->refMap->getSqlTable(con, db, tab);
  if(sqlTable.getTabStr().isEmpty()){
    KMessageBox::error(this, i18n("table not found"));
    return;
  }
  QString err = sqlTable.getError();
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }           

  QueryInput qi;

  qi.setQueryName(i18n("Insert"));
  qi.setTabStr(tab);
  qi.setDbStr(db);
  qi.setConStr(con);
  qi.setShowQuery(false);
  qi.setType(QueryInput::Insert);

  QString query = "insert into " + tab + " ";
  int fieldNum = sqlTable.getFieldNum();

  QStringList valList;
  
  for(int i = 0; i < fieldNum; i++){
    
    QString realDef = "";
    QString def = sqlTable.mapList["Default"][i];
      if(def.isEmpty() || def.at(0) == '\'' || def == "NULL")
        realDef = def;
      else
        realDef += "'" + def + "'";
        
    QString val = "<input \"" + sqlTable.mapList["Name"][i] + "\" \"" + sqlTable.mapList["Type"][i] + "\">" + realDef + "</input>";
    valList.append(val);
  }
  
  query += valList.join(" ");
  
  qi.setQuery(query);
  
  if(!doc->sendQuery(qi))
    KMessageBox::error(this, doc->getError());
}
void KSqlShellApp::slotListTabShowColumns(){
  QString tab = view->naviList->currentItem()->text(0);
  QString db =  view->naviList->currentItem()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->text(0);

  SqlTable sqlTable = doc->refMap->getSqlTable(con, db, tab);
  if(sqlTable.getTabStr().isEmpty()){
    KMessageBox::error(this, i18n("table not found"));
    return;
  }
  QString err = sqlTable.getError();
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }           
  sqlTable.setConStr(con);
  
  view->cwin->showTableTab(doc, sqlTable);
}
void KSqlShellApp::slotListTabChange(){
  
}

void KSqlShellApp::slotListTabDel(){
  QString tab = view->naviList->currentItem()->text(0);
  QString db = view->naviList->currentItem()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->text(0);
  QString msgStr = i18n("drop table ") + tab + "?";
  
  if(KMessageBox::warningYesNo(this, msgStr) == 4)
    return;

  SqlTable sqlTable = doc->refMap->getSqlTable(con, db, tab);
  sqlTable.setConStr(con);
  sqlTable.setTableAction(SqlTable::DelTable);
  
  if(!doc->makeTable(sqlTable))
     KMessageBox::error(this, doc->getError());
  slotSetActionStatus();
}

void KSqlShellApp::slotListFieldChange(){
  QString field = view->naviList->currentItem()->text(0);
  QString tab = view->naviList->currentItem()->parent()->text(0);
  QString db = view->naviList->currentItem()->parent()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->parent()->text(0);
  
  SqlTable table = doc->refMap->getSqlTable(con, db, tab);
  table.setTableAction(SqlTable::ChangeField);
  table.setChangeField(field);
  
  FieldDlg fieldDlg(doc, table, this);
  fieldDlg.exec();
}
void KSqlShellApp::slotListFieldDel(){
  QString field = view->naviList->currentItem()->text(0);
  QString tab = view->naviList->currentItem()->parent()->text(0);
  QString db = view->naviList->currentItem()->parent()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->parent()->text(0);
  
  QString msgStr = i18n("drop field ") + field + "?";
  if(KMessageBox::warningYesNo(this, msgStr) == 4)
    return;

  SqlTable table = doc->refMap->getSqlTable(con, db, tab);
  table.setTableAction(SqlTable::DelField);
  table.setChangeField(field);
  
  if(!doc->makeTable(table))
     KMessageBox::error(this, doc->getError());
  
  slotSetActionStatus();
}
void KSqlShellApp::slotSetup(){
  SetupDlg sDlg(doc, this, i18n("Setup"));
  sDlg.exec();
  if(view->cwin->queryList){
    view->cwin->qListTab->slotSyncList();
    view->cwin->qListTab->slotDelText();
  }
}
void KSqlShellApp::slotRefresh(){
  doc->setNoSyncNaviList(false);
  doc->loadNaviListData();
}
void KSqlShellApp::setActionEnabled(int status, const QString & con){

   refresh->setEnabled(actionStatus[0][status]); 
   newConnect->setEnabled(actionStatus[1][status]);
   conDel->setEnabled(actionStatus[2][status]);
   conCon->setEnabled(actionStatus[3][status]);
   conInfo->setEnabled(actionStatus[4][status]);
   conDiscon->setEnabled(actionStatus[5][status]);
   conNewDb->setEnabled(actionStatus[6][status]);
   dbDel->setEnabled(actionStatus[7][status]);
   dbNewTab->setEnabled(actionStatus[8][status]);
   tabRename->setEnabled(actionStatus[9][status]);
   tabDel->setEnabled(actionStatus[10][status]);
   tabNewField->setEnabled(actionStatus[11][status]);
   //tabChange->setEnabled(actionStatus[8][status]);
   tabSelect->setEnabled(actionStatus[12][status]);
   tabSelectAll->setEnabled(actionStatus[13][status]);
   tabInsert->setEnabled(actionStatus[14][status]);
   tabShowColumns->setEnabled(actionStatus[15][status]);
   fieldRename->setEnabled(actionStatus[17][status]);
   
   if(doc->clQueueMap.contains(con) && doc->clQueueMap[con]->getType() == ClQueue::PGSQL){
    fieldChange->setEnabled(false);
    if(doc->clQueueMap[con]->getVersionNum() >= 7.3)
      fieldDel->setEnabled(actionStatus[18][status]);
    else
      fieldDel->setEnabled(false);
   }else{
    fieldChange->setEnabled(actionStatus[16][status]);
    fieldDel->setEnabled(actionStatus[18][status]);
   }
   
   saveQuery->setEnabled(actionStatus[19][status]);
   editQuerys->setEnabled(actionStatus[20][status]);
}
                                       
void KSqlShellApp::slotSetActionStatus(){

  QListViewItem *item = view->naviList->currentItem();

  doc->setSyncNaviList(true);
  view->naviList->slotItemRenameEnd();
  
  if(!item){
    setActionEnabled(0, "");
    return;
  }
    
  int depth = item->depth();

  QString con = view->naviList->getCon(item, depth);
  
  if(!view->naviList->isSelected(item))
    setActionEnabled(0, con);
  else if(depth == 0)
    if(item->text(1).isEmpty())
      setActionEnabled(1, con); 
    else
      setActionEnabled(2, con);
      
  else if(depth == 1){
    setActionEnabled(3, con);
 
  }else if(depth == 2)
    setActionEnabled(4, con);
  else if(depth == 3)
    setActionEnabled(5, con);
  else
    setActionEnabled(0, con);

  view->naviList->slotUpdateList();  
}
void KSqlShellApp::slotStartRenameListItem(){
  oldItemName = view->naviList->currentItem()->text(0);
  view->naviList->slotRenameListItem();
  doc->setSyncNaviList(false);
}
void KSqlShellApp::slotListTabNewField(){
  QString tab = view->naviList->currentItem()->text(0);
  QString db =  view->naviList->currentItem()->parent()->text(0);
  QString con = view->naviList->currentItem()->parent()->parent()->text(0);

  SqlTable table = doc->refMap->getSqlTable(con, db, tab);
  if(table.getTabStr().isEmpty()){
    KMessageBox::error(this, i18n("table not found"));
    return;
  }
  QString err = table.getError();
  if(!err.isEmpty()){
    KMessageBox::error(this, err);
    return;
  }
  
  table.setTableAction(SqlTable::AddField);
    
  FieldDlg fieldDlg(doc, table, this);
  fieldDlg.exec();
}
void KSqlShellApp::slotRenameListItem(QListViewItem * item, int d, const QString & newName){
  //qDebug("%s  %d %s", item->text(0).latin1(), d, newName.latin1());
  if(!item){
    doc->setSyncNaviList(true);
    return;
  }
  
  QString conStr = "";
  QString dbStr = "";
  QString tabStr = "";
  QString oldTabStr = "";
  QString msgStr = "";

  if(item->depth() == 2){
    tabStr = item->text(0);
    oldTabStr = oldItemName;
    dbStr = item->parent()->text(0);
    conStr = item->parent()->parent()->text(0);
    msgStr = i18n("rename table ") + oldItemName + i18n(" to ")  + newName;
  }else if(item->depth() == 3){
    tabStr = item->parent()->text(0);
    oldTabStr = tabStr;
    dbStr = item->parent()->parent()->text(0);
    conStr = item->parent()->parent()->parent()->text(0);
    msgStr = i18n("rename field ") +  oldItemName + i18n(" to ")  + newName;
  }
  
  if(KMessageBox::warningYesNo(this, msgStr) == 4){
    item->setText(0, oldItemName);
    doc->setSyncNaviList(true);
    return;
  }
  
  SqlTable table = doc->refMap->getSqlTable(conStr, dbStr, oldTabStr);

  if(item->depth() == 2){
    table.setTableAction(SqlTable::RenameTable);
    table.setNewTabName(newName);
  }else if(item->depth() == 3){
    table.setTableAction(SqlTable::RenameField);
    table.setChangeField(oldItemName);
    table.mapList["Name"].append(newName);
  }

  if(!doc->makeTable(table)){
    KMessageBox::error(this, doc->getError());
    item->setText(0, oldItemName);
  }
  
  doc->setSyncNaviList(true);
}
void KSqlShellApp::slotMovieClicked(){
  //qDebug("jo");
}