/***************************************************************************
                          ksqlshell.h  -  description
                             -------------------
    begin                : Mit Feb 26 16:50:35 CET 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSQLSHELL_H
#define KSQLSHELL_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

#include <qlistview.h>
#include <qvaluevector.h>
#include "clqueue.h"

// forward declaration of the KSqlShell classes
class KSqlShellDoc;
class KSqlShellView;

/**
  * The base class for KSqlShell application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KSqlShellView creates your center view, which is connected
  * to the window's Doc object.
  * KSqlShellApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 1.2 code generation
  */

static const bool actionStatus[21][6] = {
      //refresh
      {true, true, true, true, true, true},
      //new connection
      {true, true, true, true, true, true},
      //del connection
      {false, true, false, false, false, false},
      //connect
      {false, true, false, false, false, false},
      //info
      {false, false, true, false, false, false},
      //disconnect
      {false, false, true, false, false, false},
      //new database
      {false, false, true, false, false, false},
      //del database
      {false, false, false, true, false, false},
      //new table
      {false, false, false, true, false, false},
      //rename table
      {false, false, false, false, true, false},
      //del table
      {false, false, false, false, true, false},
      //new field
      {false, false, false, false, true, false},
      //change tab
      //{false, false, false, false, true, false},
      //select
      {false, false, false, false, true, false},
      //select all
      {false, false, false, false, true, false},
      //insert
      {false, false, false, false, true, false},
      //show columns
      {false, false, false, false, true, false},
      //change field
      {false, false, false, false, false, true},
      //rename field
      {false, false, false, false, false, true},
      //del field
      {false, false, false, false, false, true},
      //save query
      {false, false, true, true, true, true},
      //edit querys
      {true, true, true, true, true, true}
};
  class KSqlShellApp : public KMainWindow
{
  Q_OBJECT

  friend class KSqlShellView;

  public:
    /** construtor of KSqlShellApp, calls all init functions to create the application.
     */
    KSqlShellApp(QWidget* parent=0, const char* name=0);
    ~KSqlShellApp();
    /** opens a file specified by commandline option
     */
    void openDocumentFile(const KURL& url=0);
    /** returns a pointer to the current document connected to the KTMainWindow instance and is used by
     * the View class to access the document object's methods
     */	
    KSqlShellDoc *getDocument() const; 	
     /** the configuration object of the application */
    KConfig *config;

    
  protected:
    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */ 	
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initializes the KActions of the application */
    void initActions();
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initNaviListActions();

    void initQueryMenu(QListViewItem *item);
 
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
    /** creates the centerwidget of the KTMainWindow instance and sets it as the view
     */
    void initView();
    /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
    /** saves the window properties for each open window during session end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     * @see KTMainWindow#saveProperties
     */
    virtual void saveProperties(KConfig *_cfg);
    /** reads the session config file and restores the application's state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     * @see KTMainWindow#readProperties
     */
    virtual void readProperties(KConfig *_cfg);

  public slots:
    /** open a new application window by creating a new instance of KSqlShellApp */
    void slotFileNewWindow();
    /** clears the document in the actual view to reuse it as the new document */
    void slotFileNew();
    /** open a file and load it into the document*/
    void slotFileOpen();
    /** opens a file from the recent files menu */
    void slotFileOpenRecent(const KURL& url);
    /** save a document */
    void slotFileSave();
    /** save a document by a new filename*/
    void slotFileSaveAs();
    /** asks for saving if the file is modified, then closes the actual file and window*/
    void slotFileClose();
    /** print the actual file */
    void slotFilePrint();
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();
    /** put the marked text/object into the clipboard and remove
     *	it from the document
     */
    void slotEditCut();
    /** put the marked text/object into the clipboard
     */
    void slotEditCopy();
    /** paste the clipboard into the document
     */
    void slotEditPaste();
    /** toggles the toolbar
     */
    void slotViewToolBar();
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);

    void slotNewConnect();
    void slotSaveQuery();
    void slotEditQueryList();
    void slotSendQuery(int id);

    void slotListPopupMenu(QListViewItem* listItem, const QPoint &p, int con);
    
    void slotSetActionStatus();

    void slotRunMovie(bool run);
  private slots:
    void slotListConCon();
    void slotListConDiscon();
    void slotListConDel();
    void slotListConNewDb();
    void slotListDbNewTab();
    void slotListDbDel();
    void slotListConInfo();
    void slotListTabSelect();
    void slotListTabSelectAll();
    void slotListTabInsert();
    void slotListTabShowColumns();
    void slotListTabChange();
    void slotListTabDel();
    void slotListTabNewField();
    void slotListFieldChange();
    void slotListFieldDel();
    void slotSetup();
    void slotRefresh();
    void slotStartRenameListItem();
    void slotRenameListItem(QListViewItem * item, int d, const QString & newName);
    void slotMovieClicked();

    
  private:
    
    /** view is the main widget which represents your working area. The View
     * class should handle all events of the view widget.  It is kept empty so
     * you can create your view according to your application's needs by
     * changing the view class.
     */
      
    KSqlShellView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    KSqlShellDoc *doc;

    // KAction pointers to enable/disable actions
    KAction* fileNewWindow;
    KAction* fileNew;
    KAction* fileOpen;
    KRecentFilesAction* fileOpenRecent;
    KAction* fileSave;
    KAction* fileSaveAs;
    KAction* fileClose;
    KAction* filePrint;
    KAction* fileQuit;
    KAction* editCut;
    KAction* editCopy;
    KAction* editPaste;
    
    KAction* newConnect;
    KAction* saveQuery;
    KAction* editQuerys;
    
    KAction* conCon;
    KAction* conDiscon;
    KAction* conDel;
    KAction* conNewDb;
    KAction* conInfo;
    
    KAction* dbNewTab;
    KAction* dbDel;

    KAction* tabSelect;
    KAction* tabSelectAll;
    KAction* tabInsert;
    KAction* tabShowColumns;
    KAction* tabRename;
    KAction* tabChange;
    KAction* tabDel;
    KAction* tabNewField;

    KAction* fieldDel;
    KAction* fieldRename;
    KAction* fieldChange;

    KAction* setup;
    KAction* refresh;
    
    void setActionEnabled(int status, const QString & con);
    
    KToggleAction* viewToolBar;
    KToggleAction* viewStatusBar;
    
    QPopupMenu *conMenu;
    QPopupMenu *queryMenu;
    

    QPopupMenu *listConMenu;
    QPopupMenu *listDbMenu;
    QPopupMenu *listTabMenu;
    QPopupMenu *listFieldMenu;

    QPopupMenu *queryListMenu;
    QPopupMenu *viewMenu;

    QPopupMenu *setupMenu;
    
    QString oldItemName;

    int movieTbId;
};
 
#endif // KSQLSHELL_H
