/***************************************************************************
                          ksqlshelldoc.cpp  -  description
                             -------------------
    begin                : Mit Feb 26 16:50:35 CET 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>

// include files for KDE
#include <klocale.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>

// application specific includes
#include "ksqlshelldoc.h"
#include "ksqlshell.h"
#include "ksqlshellview.h"
//test
#include <unistd.h>

QList<KSqlShellView> *KSqlShellDoc::pViewList = 0L;

KSqlShellDoc::KSqlShellDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  if(!pViewList)
  {
    pViewList = new QList<KSqlShellView>();
  }

  pViewList->setAutoDelete(true);

  refMap = new ReferenceMap();
  refMap0 = new ReferenceMap();
  error = "";
  syncNaviListTime = 20;
  loadNaviListDataTimerId = 0;
  syncNaviListTimerId = 0;
  resultTimerId = 0;
  sync = true;
  noSync = false;
}

KSqlShellDoc::~KSqlShellDoc()
{
}

void KSqlShellDoc::setSyncNaviListTime(int sec){

  if(syncNaviListTimerId)
    killTimer(syncNaviListTimerId);
  if(sec > 0)
    syncNaviListTime = sec;  
  syncNaviListTimerId = startTimer(syncNaviListTime * 1000);
  
}
KConfig * KSqlShellDoc::getConfig(){
  return ((KSqlShellApp *)this->parent())->config; 
}
bool KSqlShellDoc::updateConnection(const QString & conStr){
  if(!clQueueMap.contains(conStr)){
    error = conStr + "don't exist";
    return false;
  }
  if(!clQueueMap[conStr]->isConnect())
    refMap->removeConnection(conStr);
    
  else if(!refMap->contains(conStr)){
    SqlClient *sqlCl;
    if(!(sqlCl = clQueueMap[conStr]->getCl())){
      error = clQueueMap[conStr]->getError();
      return false;
    }
    refMap->insertConnection(conStr, sqlCl);
    if(!refMap->loadConnection(conStr)){
      error = refMap->getError();
      return false;
    }

  }
  noSync = true;
  refMap->syncConnection(pViewList->at(0)->naviList->findNaviItem(conStr));
  return true;
}
bool KSqlShellDoc::updateDatabase(const QString & conStr, const QString & dbStr){
  if(!refMap->loadConnection(conStr)){
    error = refMap->getError();
    return false;
  }
  
  noSync = true;
  refMap->syncDatabase(pViewList->at(0)->naviList->findNaviItem(conStr),pViewList->at(0)->naviList->findNaviItem(conStr, dbStr), dbStr);
  return true;
}
bool KSqlShellDoc::updateTable(const QString & conStr, const QString & dbStr, const QString & tabStr){
   if(!refMap->loadDatabase(conStr, dbStr)){
    error = refMap->getError();
    return false;
  }
  noSync = true;
  refMap->syncTable(pViewList->at(0)->naviList->findNaviItem(conStr, dbStr),pViewList->at(0)->naviList->findNaviItem(conStr, dbStr, tabStr), tabStr);
  return true;
}
bool KSqlShellDoc::updateField(const QString & conStr, const QString & dbStr, const QString & tabStr, const QString & fieldStr){
   if(!refMap->loadTable(conStr, dbStr, tabStr)){
    error = refMap->getError();
    return false;
  }
  noSync = true;
  refMap->syncField(pViewList->at(0)->naviList->findNaviItem(conStr, dbStr, tabStr), pViewList->at(0)->naviList->findNaviItem(conStr, dbStr, tabStr,fieldStr), fieldStr);
  return true;
}
void KSqlShellDoc::loadNaviListData(){
  if(!refMap0->loadEnd()) {
    return;
  }

  mapIt = clQueueMap.begin();
  for(;mapIt != clQueueMap.end(); ++mapIt){

    if(mapIt.data()->isConnect() && !refMap0->contains(mapIt.key())){
      SqlClient *sqlCl;
      if(!(sqlCl = mapIt.data()->getCl())){
        error = mapIt.data()->getError();
        disconnectMsg(mapIt.key());
        continue;
      }
      refMap0->insertConnection(mapIt.key(), sqlCl);

    }else if(!mapIt.data()->isConnect() && refMap0->contains(mapIt.key()))
      refMap0->removeConnection(mapIt.key());

  }

  refMap0->load();
  emit signalRunSync(true);
  if(!loadNaviListDataTimerId){
    loadNaviListDataTimerId = startTimer(500);

  }
}
void KSqlShellDoc::syncNaviList(){
  if(!sync)
    return;

  if(noSync){
    noSync = false;
  }else{
    ReferenceMap *refMapTemp = refMap;
    refMap = refMap0;
    refMap0 = refMapTemp;
    refMap->sync(pViewList->at(0)->naviList);

  }
    
}
void KSqlShellDoc::getResult(const QString & conStr){
  ClQueue * clQ = clQueueMap[conStr];
  
  SqlClient * sqlCl;
  while((sqlCl = clQ->getResultCl())){
    if(sqlCl->queryOk() && !sqlCl->isResult()){
      clQ->freeCl(sqlCl);
      continue;
    }
    pViewList->at(0)->cwin->showResultTab(this, sqlCl);
  }
    
}
void KSqlShellDoc::timerEvent(QTimerEvent *e){
  if(e->timerId() == syncNaviListTimerId){
    loadNaviListData();
    
  }else if(e->timerId() == loadNaviListDataTimerId){

    if(refMap0->loadEnd()){
     
      killTimer(loadNaviListDataTimerId);
      loadNaviListDataTimerId = 0;
      syncNaviList();
      emit signalRunSync(false);
    }
       
  }else if(e->timerId() == resultTimerId){
    bool running = false;

    mapIt = clQueueMap.begin();
    for(;mapIt != clQueueMap.end(); ++mapIt){

      if(!mapIt.data()->isRunning())
        continue;

      getResult(mapIt.key());
        
      running = true;
    }

    if(!running){
      killTimer(resultTimerId);
      resultTimerId = 0;
    }
  }
}

void KSqlShellDoc::addView(KSqlShellView *view)
{
  pViewList->append(view);
}

void KSqlShellDoc::removeView(KSqlShellView *view)
{
  pViewList->remove(view);
}
void KSqlShellDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KSqlShellDoc::URL() const
{
  return doc_url;
}

void KSqlShellDoc::slotUpdateAllViews(KSqlShellView *sender)
{
  KSqlShellView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool KSqlShellDoc::saveModified()
{
  bool completed=true;

  if(modified)
  {
    KSqlShellApp *win=(KSqlShellApp *) parent();
    int want_save = KMessageBox::warningYesNoCancel(win,
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"),
                                         i18n("Warning"));
    switch(want_save)
    {
      case KMessageBox::Yes:
           if (doc_url.fileName() == i18n("Untitled"))
           {
             win->slotFileSaveAs();
           }
           else
           {
             saveDocument(URL());
       	   };

       	   deleteContents();
           completed=true;
           break;

      case KMessageBox::No:
           setModified(false);
           deleteContents();
           completed=true;
           break;

      case KMessageBox::Cancel:
           completed=false;
           break;

      default:
           completed=false;
           break;
    }
  }

  return completed;
}

void KSqlShellDoc::closeDocument()
{
  deleteContents();
}

bool KSqlShellDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  doc_url.setFileName(i18n("Untitled"));

  return true;
}

bool KSqlShellDoc::openDocument(const KURL& url, const char *format /*=0*/)
{
  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile );
  /////////////////////////////////////////////////
  // TODO: Add your document opening code here
  /////////////////////////////////////////////////

  KIO::NetAccess::removeTempFile( tmpfile );

  modified=false;
  return true;
}

bool KSqlShellDoc::saveDocument(const KURL& url, const char *format /*=0*/)
{
  /////////////////////////////////////////////////
  // TODO: Add your document saving code here
  /////////////////////////////////////////////////

  modified=false;
  return true;
}

void KSqlShellDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool KSqlShellDoc::testConnect(ClQueue::DbType dbType,
                                                    const QString & dbHost,
                                                    int dbPort,
                                                    const QString & dbUser,
                                                    const QString & dbPw,
                                                    const QString & dbName)
{
  ClQueue clQueue;
  clQueue.initQueue(dbType, dbHost, dbPort, dbUser, dbPw, dbName);
  
  if(!clQueue.connect(1)){
    error = clQueue.getError();
    return false;
  }

  clQueue.disconnect();
  return true;
}

bool KSqlShellDoc::addConnect(ClQueue::DbType dbType,
                                                    const QString & dbHost,
                                                    int dbPort,
                                                    const QString & dbUser,
                                                    const QString & dbPw,
                                                    const QString & dbName)
{
  //init key
  QString conStr = ClQueue::getDbTypeStr(dbType) + " " + dbUser + "@" + dbHost;
  QString newConStr = conStr;
  int i = 2;
  mapIt = clQueueMap.find(conStr);

  //new key if exist
  while(mapIt != clQueueMap.end()){
    newConStr = conStr + " " + QString::number(i++);
    mapIt = clQueueMap.find(newConStr);
  }

  //init clQueue and add to clQueueMap
  ClQueue *pClQ = new ClQueue();
  pClQ->initQueue(dbType, dbHost, dbPort, dbUser, dbPw, dbName);
  clQueueMap[newConStr] = pClQ;
  
  // add clQueue key to naviList
  pViewList->at(0)->naviList->addConnect(dbType, newConStr);
  
  return true;
}

void KSqlShellDoc::delConnect(const QString & conStr){
  delete clQueueMap[conStr];
  clQueueMap.remove(conStr);
  pViewList->at(0)->naviList->delConnect(conStr);
}

bool KSqlShellDoc::connect(const QString & conStr){
  if(!clQueueMap[conStr]->connect(3)){
    error =  clQueueMap[conStr]->getError();
    return false;
  }
  
  if(!updateConnection(conStr))
    return false;
  
  return true;
}
void KSqlShellDoc::disconnect(const QString & conStr){
  clQueueMap[conStr]->disconnect();
  updateConnection(conStr);
}
void KSqlShellDoc::disconnectMsg(const QString & conStr){
 disconnect(conStr);
 ((KSqlShellApp *)parent())->slotSetActionStatus();
 KMessageBox::error((KSqlShellApp *)parent(), getError()); 
}
bool KSqlShellDoc::addDb(const QString & conStr, const QString & db, const QString & templateDb){
  if(!clQueueMap[conStr]->createDb(db, templateDb)){
    error = clQueueMap[conStr]->getError();
    return false;
  }
  if(!updateDatabase(conStr, db))
    return false;

  return true;  
}
bool KSqlShellDoc::delDb(const QString & conStr, const QString & db){
  if(!clQueueMap[conStr]->dropDb(db)){
    error = clQueueMap[conStr]->getError();
    return false;
  }
  if(!updateDatabase(conStr, db))
    return false;
  return true;
}

bool KSqlShellDoc::makeTable(SqlTable & table){
  QString conStr = table.getConStr();
  QString dbStr = table.getDbStr();
  QString tabStr = table.getTabStr();

  if(!clQueueMap[conStr]->makeTable(table)){
    error = clQueueMap[conStr]->getError();
    return false;
  }

  SqlTable::TableAction action = table.getTableAction();
  
  if(action == SqlTable::NewTable || action == SqlTable::RenameTable || action == SqlTable::DelTable){
    if(!updateTable(conStr, dbStr, tabStr))
      return false;
  }else if(action == SqlTable::AddField){
    //qDebug("cf: %s , t: %s:", table.mapList["Name"][table.getFieldNum()].latin1(), table.mapList["Type"][table.getFieldNum()].latin1());
    if(!updateField(conStr, dbStr, tabStr, table.mapList["Name"][table.getFieldNum()]))
      return false;
  }else{
    if(!updateField(conStr, dbStr, tabStr, table.getChangeField()))
      return false;
  }
  return true;
  
}

bool KSqlShellDoc::sendQuery(QueryInput & queryInput){
  if(!clQueueMap.contains(queryInput.getConStr())){
    error = "'" + queryInput.getConStr() + "' don't exist";
    return false;
  }

  if(!queryInput.startQuery())
    return true;
  
  if(!clQueueMap[queryInput.getConStr()]->sendQuery(queryInput)){
    error = clQueueMap[queryInput.getConStr()]->getError();
    return false;
  }
  
  if(!resultTimerId){
    resultTimerId = startTimer(500);
  
    if(resultTimerId  == 0){
      error = i18n("no result timer");
      return false;
    }
  }
  
  return true;
}

QString KSqlShellDoc::getError(){
  QString err = error;
  error = "";
  return err;
}
