/***************************************************************************
                          ksqlshelldoc.h  -  description
                             -------------------
    begin                : Mit Feb 26 16:50:35 CET 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSQLSHELLDOC_H
#define KSQLSHELLDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>
#include <qmap.h>
#include <qstrlist.h>
#include <qstringlist.h>
#include <kconfig.h>
#include "sqltable.h"
#include "queryinput.h"
#include "clqueue.h"
#include "referencemap.h"
#include "querymap.h"
// include files for KDE
#include <kurl.h>



// forward declaration of the KSqlShell classes
class KSqlShellView;

/**	KSqlShellDoc provides a document object for a document-view model.
  *
  * The KSqlShellDoc class provides a document object that can be used in conjunction with the classes KSqlShellApp and KSqlShellView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the KSqlShellApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KSqlShellView objects. Also, KSqlShellDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */

class KSqlShellDoc : public QObject
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    KSqlShellDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KSqlShellDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KSqlShellView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KSqlShellView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** "save modified" - asks the user for saving if the document is modified */
    bool saveModified();	
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const KURL& url, const char *format=0);
    /** saves the document under filename and format.*/	
    bool saveDocument(const KURL& url, const char *format=0);
    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
	  void setURL(const KURL& url);

    
    bool testConnect(ClQueue::DbType dbType, const QString & dbHost, int dbPort, const QString & dbUser, const QString & dbPw, const QString & dbName);
    bool addConnect(ClQueue::DbType dbType, const QString & dbHost, int dbPort, const QString & dbUser, const QString & dbPw, const QString & dbName);
    void delConnect(const QString & conStr);

    bool connect(const QString & conStr);
    void disconnect(const QString & conStr);

    bool addDb(const QString & conStr, const QString & db, const QString & templateDb = "");
    bool delDb(const QString & conStr, const QString & db);

    bool makeTable(SqlTable & table);
    
    bool sendQuery(QueryInput & queryInput);
    
   QString getError();

  signals:

    void  signalRunSync( bool );
     
  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(KSqlShellView *sender);
 	 
  public:	
    /** the list of the views currently connected to the document */
    static QList<KSqlShellView> *pViewList;	

    QMap<QString,ClQueue*> clQueueMap;
    QMap<QString,ClQueue*>::iterator mapIt;

    ReferenceMap *refMap, *refMap0;

    QueryMap queryMap;

    void setSyncNaviListTime(int sec);
    int getSyncNaviListTime() { return syncNaviListTime;};

    void setSyncNaviList(bool s) { sync = s;};
    void setNoSyncNaviList(bool s)  {noSync = s;};
    void loadNaviListData();
    void syncNaviList();
    
    KConfig * getConfig();
  
  private:
    /** the modified flag of the current document */
    bool modified;
    KURL doc_url;

    void timerEvent(QTimerEvent *);
    
    
    void getResult(const QString & conStr);
    void disconnectMsg(const QString & conStr);

    bool updateConnection(const QString & conStr);
    bool updateDatabase(const QString & conStr, const QString & dbStr);
    bool updateTable(const QString & conStr, const QString & dbStr, const QString & tabStr);
    bool updateField(const QString & conStr, const QString & dbStr, const QString & tabStr, const QString & fieldStr);

    int syncNaviListTime;
    int loadNaviListDataTimerId;
    int syncNaviListTimerId;
    int resultTimerId;

    bool sync;
    bool noSync;
    
  ClQueue clQueue;
  QString error;  
};

#endif // KSQLSHELLDOC_H
