/***************************************************************************
                          mysqlcl.cpp  -  description
                             -------------------
    begin                : Mon Mr 17 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mysqlcl.h"
#include <klocale.h>

MyResult::MyResult(const QString & st) : Result(st){
  res = 0;
}
MyResult::~MyResult(){
  if(res)
    mysql_free_result(res);
}
void MyResult::setResult(void *result){
  res = (MYSQL_RES *)result;
  if(res)
    isRes = true;
}
int MyResult::getColNum(){
  return (int)mysql_num_fields(res); 
}
int MyResult::getRowNum(){
  return (int) mysql_num_rows(res);
}
QString MyResult::getColName(int colnum){
  MYSQL_FIELD *field = mysql_fetch_field_direct(res, colnum);
  return QString(field->name);
}
bool MyResult::nextRow(){
  if((row = mysql_fetch_row(res)))
    return true;
  return false;
}
char * MyResult::getValue(int colnum){
  char * field = row[colnum];
  if(!field)
    field = "NULL";
  return field;
}
void MyResult::setFirstRow(){
  if(res)
    mysql_data_seek(res, 0);
}


MySqlCl::MySqlCl(){
}

MySqlCl::MySqlCl(const QString & dbHost, int dbPort, const QString & dbUser, const QString & dbPw, const QString & dbName):SqlClient(dbHost, dbPort, dbUser, dbPw, dbName){

}

MySqlCl::~MySqlCl(){
  clear();
  mysql_close(&con);
  //....
}
bool MySqlCl::initSqlCl(){

  if(!mysql_init(&con) || !mysql_real_connect(&con, host.latin1(), user.latin1(), pw.latin1(), defDb.latin1(), port ,0,0)){
    error = QString(mysql_error(&con));
    return false;
  }

  aktDb = defDb;

  return true;
}

bool MySqlCl::useDb(const QString& db){
  QString uDb = defDb;
  if(!db.isEmpty())
    uDb = db;

  if(mysql_select_db(&con, uDb.latin1())){
    error = mysql_error(&con);
    return false;
  }

  aktDb = uDb;
  return true;
}
bool MySqlCl::checkCon(){
  if(mysql_ping(&con)){
    error = mysql_error(&con);
    return false;
  }
  return true;
}
bool MySqlCl::startQuery(QStringList & stList){
  if(!checkCon())
    return false;

  for(QStringList::Iterator it = stList.begin(); it != stList.end(); ++it)
    resultList.append(new MyResult(*it));

  start();
  return true;
}
bool MySqlCl::runStmt(const QString & st, bool isRes){
  if(!checkCon())
    return false;

  if(mysql_real_query(&con, st.latin1(), st.length())){
    error = mysql_error(&con);
    return false;
  }

  if(isRes){
    MyResult *res = new MyResult(st);
    res->setResult(mysql_store_result(&con));
    resultList.append(res);
  }

  return true;
}
bool MySqlCl::getDatabases(){
  MyResult *res = new MyResult("getDatabases");
  res->setResult(mysql_list_dbs(&con, 0));
  if(res->isResult()){
    resultList.append(res);
    return true;
  }
  error = mysql_error(&con);

  return false;
}
bool MySqlCl::getTables(){
  MyResult *res = new MyResult("getTables");
  res->setResult(mysql_list_tables(&con, 0));
  if(res->isResult()){
    resultList.append(res);
    return true;
  }

  error = mysql_error(&con);

  return false;
}
bool MySqlCl::makeTable(SqlTable & table){
  QString tabStr = table.getTabStr();
  QString fieldStr = table.getChangeField();
  int fieldNum = table.getFieldNum();
  //qDebug("fieldNum: %d, nLen: %d, tLen: %d, dLen: %d, f_index: %d", fieldNum, table.mapList[i18n("Name")].count(), table.mapList[i18n("Type")].count(), table.mapList[i18n("Default")].count(), table.getFieldIndex(fieldStr));
  QString newFieldStr = table.mapList["Name"][fieldNum];
  //qDebug("fieldStr: %s", fieldStr.latin1());

  SqlTable::TableAction action =  table.getTableAction();
  QString altStmt = "alter table " + tabStr + " ";

  if(action == SqlTable::RenameField || action == SqlTable::ChangeField){

    int idx = table.getFieldIndex(fieldStr);
    if(action == SqlTable::ChangeField)
      idx =  fieldNum;
    QString type = table.mapList["Type"][idx];
    QString null = "";
    QString def = "";
    if(table.mapList["NULL"][idx].isEmpty())
      null = "not null";
    if(!table.mapList["Default"][idx].isEmpty())
      def = "DEFAULT " + table.mapList["Default"][idx];

    altStmt += "change " + fieldStr + " " + newFieldStr + " " + type + " " + null + " " + def;

  }else if(action == SqlTable::AddField){

    QString null = "";
     if(table.mapList["NULL"][fieldNum].isEmpty())
      null = "not null";
    altStmt += "add " + newFieldStr + " " +  table.mapList["Type"][fieldNum] + " " +  null + " ";
    if(!table.mapList["Default"][fieldNum].isEmpty()){
      altStmt += " DEFAULT " + table.mapList["Default"][fieldNum] + " ";
    }

    altStmt += table.getNewFieldPos();

  }

  //qDebug(altStmt.latin1());
  return runStmt(altStmt, false);

}
bool MySqlCl::createDb(const QString & db, const QString & tempDb){
  QString dbStmt = "create database " + db;
  return runStmt(dbStmt, false);
}
bool MySqlCl::getTemplates(){
  return false;
}

QString MySqlCl::limitClause(int offset, int count){
  return QString("LIMIT %1, %2").arg(offset).arg(count);
}
bool MySqlCl::getSqlTable(SqlTable * sqlTable){

  if(!useDb(sqlTable->getDbStr()))
    return false;

  setCurrentTable(sqlTable->getTabStr());

  QString colStmt = "show columns from " + table;
  if(!runStmt(colStmt))
    return false;

  Result * res  =  resultList.at(0);

  int i = 0;
  while(res->nextRow()){
    sqlTable->mapList["Name"].append(QString(res->getValue(0)));
    sqlTable->mapList["Type"].append(QString(res->getValue(1)));
    sqlTable->mapList["NULL"].append(QString(res->getValue(2)));

    if(QString(res->getValue(3)) == "PRI"){
      sqlTable->mapList["Key"].append("Primary Key");
      sqlTable->setPKey(true);
    }else
      sqlTable->mapList["Key"].append("");

    sqlTable->mapList["Default"].append((res->getValue(4)));
    sqlTable->mapList["Extra"].append(QString(res->getValue(5)));
    i++;
  }
  sqlTable->setFieldNum(i);
  sqlTable->attrList.append("Name");
  sqlTable->attrList.append("Type");
  sqlTable->attrList.append("NULL");
  sqlTable->attrList.append("Key");
  sqlTable->attrList.append("Default");
  sqlTable->attrList.append("Extra");
  
  return true;
}

bool MySqlCl::queryEnd(){
   return wait(0);
}


void MySqlCl::run(){
  MyResult * res;

  for(res = (MyResult*)resultList.first(); res; res = (MyResult*)resultList.next()){
    if(mysql_real_query(&con, res->getStmt().latin1(), res->getStmt().length())){
      res->setError(mysql_error(&con));
      exit();
    }else{
      res->setStmtOk();
      res->setResult(mysql_store_result(&con));
      if(res->isResult())
        isRes = true;
    }
  }
  queryok = true;
  exit();
}
