/***************************************************************************
                          mysqlcl.h  -  description
                             -------------------
    begin                : Mon Mr 17 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MYSQLCL_H
#define MYSQLCL_H

#include <sqlclient.h>
#include <qthread.h>
#include <mysql.h>

/**
  *@author Olaf Gawenda
  */

#define MYSQL_TYPE_NUM 25

static struct SqlType MySqlTypes[] = {
  //name, lengDec, selfdef, opt00, opt01, opt02, opt03, opt10, opt11, opt12, opt13,
  {"", false, false, "", "", "", "", "", "", "", ""},
  {"tinyint", true, false, "", "unsigned", "zerofill", "unsigned zerofilll", "", "auto_increment", "", ""},
  {"smallint", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "auto_increment", "", ""},
  {"mediumint", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "auto_increment", "", ""},
  {"int", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "auto_increment", "", ""},
  {"bigint", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "auto_increment", "", ""},
  {"double", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "", "", ""},
  {"float", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "", "", ""},
  {"decimal", true, false, "", "unsigned", "zerofill", "unsigned zerofill", "", "", "", ""},
  {"char", true,  false, "", "binary", "", "", "", "fulltext", "", ""},
  {"varchar", true, false, "", "binary", "", "","", "fulltext", "", ""},
  {"date", false, false, "", "", "", "", "", "", "", ""},
  {"time", false, false, "", "", "", "", "", "", "", ""},
  {"timestamp", false, false, "", "", "", "", "", "", "", ""},
  {"datetime", false, false, "", "", "", "", "", "", "", ""},
  {"tinyblob", false, false, "", "", "", "", "", "", "", ""},
  {"blob", false, false, "", "", "", "", "", "", "", ""},
  {"mediumblob", false, false, "", "", "", "", "", "", "", ""},
  {"longblob", false, false, "", "", "", "", "", "", "", ""},
  {"tinytext", false, false, "", "", "", "", "", "fulltext", "", ""},
  {"text", false, false, "", "", "", "", "", "fulltext", "", ""},
  {"mediumtext", false, false, "", "", "", "", "", "fulltext", "", ""},
  {"longtext", false, false, "", "", "", "", "", "fulltext", "", ""},
  {"enum", false, true, "", "", "", "", "", "", "", ""},
  {"set", false, true, "", "", "", "", "", "", "", ""}
};

class MyResult : public Result {
public:
   MyResult(const QString & st);
   ~MyResult();
   void setResult(void *result);
   int getColNum();
   int getRowNum();
   QString getColName(int colnum);
   bool nextRow();
   char * getValue(int colnum);
   void setFirstRow();
private:
    MYSQL_RES *res;
    MYSQL_ROW row;
};

class MySqlCl : public SqlClient, QThread  {
public:
  MySqlCl();
  MySqlCl(const QString& dbHost, int dbPort, const QString& dbUser, const QString& dbPw, const QString& dbName);
  ~MySqlCl();
  bool initSqlCl();
  bool useDb(const QString& db = "");
  bool startQuery(QStringList & stList);
  bool runStmt(const QString & st, bool isRes = true);
  bool getDatabases();
  bool getTables();
  bool getTemplates();
  bool makeTable(SqlTable & table);
  bool createDb(const QString & db, const QString & tempDb = "");
  bool queryEnd();
  bool getSqlTable(SqlTable * sqlTable);
  QString limitClause(int offset, int count);
  
  private:
  void run();
  bool checkCon();
  MYSQL con;
  
};

#endif
