/***************************************************************************
                          navilist.cpp  -  description
                             -------------------
    begin                : Die Mr 4 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <klocale.h>
#include <qmap.h>
#include <qvaluevector.h>
#include "navilist.h"
#include "ksqlshellsizes.h"

NaviList::NaviList(QWidget *parent, const char *name ) : QListView(parent) {

  setRootIsDecorated(true);
  addColumn(i18n("Connections"), NAVI_COL_SIZE_CON);
  addColumn(i18n("State"), NAVI_COL_SIZE_STAT);
  renameItem = 0;

  connect(this, SIGNAL(onItem(QListViewItem *)), this, SLOT(slotOnItem(QListViewItem *)));
}
NaviList::~NaviList(){
}
QString NaviList::getCon(QListViewItem *item, int depth){
  if(depth == 0)
    return item->text(0);
  if(depth == 1)
    return item->parent()->text(0);
  if(depth == 2)
    return item->parent()->parent()->text(0);
  if(depth == 3)
    return item->parent()->parent()->parent()->text(0);
  return QString("");
}
QListViewItem * NaviList::findNaviItem(const QString & conStr, const QString & dbStr, const QString & tabStr, const QString & fieldStr){
  QListViewItem  *con, *nextCon, *db, *nextDb, *tab, *nextTab, *field, *nextField;

  con = firstChild();
  while(con){
     
    if(con->text(0) != conStr){
      nextCon = con->nextSibling();
      con = nextCon;
      continue;
      
    }else if(dbStr.isEmpty())
      return con;
      
    db = con->firstChild();
    while(db){
      if(db->text(0) != dbStr){
        nextDb = db->nextSibling();
        db = nextDb;
        continue;

      }else if(tabStr.isEmpty())
          return db;
      
      tab = db->firstChild();
      while(tab){
        if(tab->text(0) != tabStr){
          nextTab = tab->nextSibling();
          tab = nextTab;
          continue;

        }else if(fieldStr.isEmpty())
          return tab;

        field = tab->firstChild();
        while(field){
          if(field->text(0) != fieldStr){
            nextField = field->nextSibling();
            field = nextField;
            continue;
          }
          return field; 
        }
        return 0; 
      }
      return 0;
    }
    return 0;
  }
  return 0;
}  


void NaviList::addConnect(ClQueue::DbType dbType, const QString &conStr){  
  QListViewItem *conItem = new QListViewItem(this, conStr);
}

void NaviList::delConnect(const QString & conStr){
  takeItem(findItem(conStr,0, ExactMatch));
  emit signalDbChanged(conStr, QString(""));
}
void NaviList::slotUpdateList(){
  
  QString con = "";
  QString db = "";
  QListViewItem *item = currentItem();
  if(!item || !isSelected(item)){
    emit signalDbChanged(con, db);
    return;
  }
  
  int depth = item->depth();
   
  if(depth == 0)
    con = item->text(0); 
  else if(depth == 1){
    con = item->parent()->text(0);
    db = item->text(0);
  }else if(depth == 2){
    con = item->parent()->parent()->text(0);
    db = item->parent()->text(0);
  }else{
    con = item->parent()->parent()->parent()->text(0);
    db = item->parent()->parent()->text(0);
  }
  
  emit signalDbChanged(con,db);
  
}

void NaviList::slotRenameListItem(){
  renameItem = currentItem();
  renameItem->setRenameEnabled(0, true);
  renameItem->startRename(0);
}
void NaviList::slotItemRenameEnd(){
  if(renameItem){
    renameItem->setRenameEnabled(0, false);
    renameItem = 0;
  }
}
void NaviList::slotOnItem(QListViewItem *item){
  QToolTip::setEnabled(true);
  QToolTip::remove(this);
  QString tip = "";
  QListViewItem *tempItem = item;
  
  for(int i = 0; i <= item->depth(); i++){
    tip = tempItem->text(0) + "  " + tip;
    tempItem = tempItem->parent();
  }
  
  QToolTip::add(this, tip);
}
