/***************************************************************************
                          pgsqlcl.cpp  -  description
                             -------------------
    begin                : Don Mr 20 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pgsqlcl.h"
#include <qstring.h>
#include <qmap.h>
#include <klocale.h>


PgResult::PgResult(const QString & st) : Result(st){
  res = 0;
  row = 0;
}
PgResult::~PgResult(){
  if(res)
    PQclear(res);
}
void PgResult::setResult(void *result){
  res = (PGresult *) result;
  if(res && PGRES_TUPLES_OK == PQresultStatus(res))
    isRes = true;
}
int PgResult::getColNum(){
  return PQnfields(res);
}
int PgResult::getRowNum(){
  return PQntuples(res);
}
QString PgResult::getColName(int colnum){
  return QString(PQfname(res, colnum));
}
bool PgResult::nextRow(){
 if(res && row <  PQntuples(res)){
    row ++;
    return true;
  }
  return false; 
}
char * PgResult::getValue(int colnum){
  return PQgetvalue(res,row -1, colnum);
}
void PgResult::setFirstRow(){
  if(res)
    row = 0;
}


PgSqlCl::PgSqlCl(){
}
PgSqlCl::PgSqlCl(const QString & dbHost, int dbPort, const QString & dbUser, const QString & dbPw, const QString & dbName):SqlClient(dbHost, dbPort, dbUser, dbPw, dbName){
}
PgSqlCl::~PgSqlCl(){
  clear();
  PQfinish(con);
}
bool PgSqlCl::initSqlCl(){
  return connect(defDb);
}
bool PgSqlCl::connect(const QString & db){
  QString portStr = QString::number(port);  
  con = PQsetdbLogin(host.latin1(), portStr.latin1(), 0, 0, db.latin1(), user.latin1(), pw.latin1());
 
 if(PQstatus(con) == CONNECTION_BAD){
    error = QString(PQerrorMessage(con));    
    return false;
 }
  aktDb = db;
  return true;
}
bool PgSqlCl::checkCon(){
  if(PQstatus(con) == CONNECTION_BAD)
    PQreset(con);

  if(PQstatus(con) == CONNECTION_BAD){
    error = QString(PQerrorMessage(con));
    return false;
  }
  return true;
}

bool PgSqlCl::useDb(const QString & db){
  PQfinish(con);
  QString uDb = defDb;
  
  if(!db.isEmpty())
    uDb = db;
    
  if(!connect(uDb))
    return false;

  aktDb = uDb;
  
  return true;
}
bool PgSqlCl::startQuery(QStringList & stList){
  if(!checkCon())
    return false;


  for(QStringList::Iterator it = stList.begin(); it != stList.end(); ++it)
    resultList.append(new PgResult(*it));
    
  start();
  
  return true;
}
bool PgSqlCl::runStmt(const QString & st, bool isRes){
  if(!checkCon())
    return false;
  
  ExecStatusType stat;
    
  PGresult *result = PQexec(con, st.latin1());
    
  stat = PQresultStatus(result);
  if(!result || stat == PGRES_FATAL_ERROR || stat == PGRES_NONFATAL_ERROR || stat == PGRES_BAD_RESPONSE){
    error = QString(PQresultErrorMessage(result));
    PQclear(result);
    return false;
  }

  if(isRes){
    PgResult * res = new PgResult(st);
    res->setResult(result);
    resultList.append(res);
  }else
    PQclear(result);
    
  return true;
}
bool PgSqlCl::getDatabases(){
  QString dbStmt = "select datname from pg_database,pg_user where datdba = usesysid and usename = '" + user + "'";
  return runStmt(dbStmt);
}
bool PgSqlCl::getTables(){
  QString tabStmt = "select relname from pg_class,pg_user where relkind in ('r') and relowner = usesysid and relname !~ '^pg_' and usename = '" + user + "'";
  return runStmt(tabStmt);
}

bool PgSqlCl::getTemplates(){
  QString tempStmt = "select datname from pg_database where datistemplate = 't'";
  return runStmt(tempStmt);
}

bool PgSqlCl::makeTable(SqlTable & table){
  
  QString tabStr = table.getTabStr();
  QString fieldStr = table.getChangeField();
  QString newFieldStr = table.mapList["Name"][table.getFieldNum()];
  QString altStmt = "alter table " + tabStr;
  QString altStmt2 = "";

  if(table.getTableAction() == SqlTable::RenameField){

    altStmt += " rename " + fieldStr + " to " + newFieldStr;

  }else if(table.getTableAction() == SqlTable::AddField){

    altStmt += " add " + newFieldStr + " " +  table.mapList["Type"][table.getFieldNum()];

    QString def =  table.mapList["Default"][table.getFieldNum()];

    if(!def.isEmpty())
      altStmt2 = "alter table " + tabStr + " alter " + newFieldStr + " set default " + def;

  }

  //qDebug(altStmt.latin1());
  //qDebug(altStmt2.latin1());

  if(!runStmt(altStmt, false))
    return false;
  if(!altStmt2.isEmpty() && !runStmt(altStmt2, false))
    return false;

  return true;
}
bool PgSqlCl::createDb(const QString & db, const QString & tempDb){
  QString stmt = "create database " + db;
  if(!tempDb.isEmpty())
    stmt += " with template = " + tempDb;

  return runStmt(stmt, false);
}

QString PgSqlCl::limitClause(int offset, int count){
  return QString("OFFSET %1 LIMIT %2").arg(offset).arg(count);
}
bool PgSqlCl::getSqlTable(SqlTable * sqlTable){
  if(!useDb(sqlTable->getDbStr()))
    return false;

  setCurrentTable(sqlTable->getTabStr());

  QString pKeyTable = table + "_pkey";

  QString colStmt = "SELECT c.relname, a.attname ,format_type(a.atttypid, a.atttypmod), a.attnotnull, d.adsrc, c.relhasoids ";
  colStmt += "FROM pg_class c, pg_attribute a LEFT JOIN pg_attrdef d ON a.attnum = d.adnum and a.attrelid = d.adrelid ";
  colStmt += "WHERE c.relname in ('" + table + "', '" + pKeyTable + "') and a.atttypid != 0 and  a.attnum > 0 and a.attrelid = c.oid order by a.attnum";

   if(!runStmt(colStmt))
    return false;

  Result * res  =  resultList.at(0);

  int i = 0;
  QMap<QString, int> indexMap;

  while(res->nextRow()){
    QString relname = QString(res->getValue(0));
    if(relname == pKeyTable)
      continue;

    if(QString(res->getValue(5)) == "t")
      sqlTable->setOid(true);

    QString name =  QString(res->getValue(1));
    indexMap[name] = i;

    sqlTable->mapList["Name"].append(name);
    sqlTable->mapList["Type"].append(QString(res->getValue(2)));
    QString null = QString(res->getValue(3));
    if(null == "t")
      sqlTable->mapList["NULL"].append("");
    else
      sqlTable->mapList["NULL"].append("YES");

    sqlTable->mapList["Key"].append("");

    QString def = QString(res->getValue(4));
    if(def.isEmpty())
      sqlTable->mapList["Default"].append("");
    else
      sqlTable->mapList["Default"].append(def);

    i++;
  }

  res->setFirstRow();


  while(res->nextRow()){
    QString relname = QString(res->getValue(0));
    if(relname == table)
      continue;

    QString name =  QString(res->getValue(1));
    if(indexMap.contains(name)){
      sqlTable->mapList["Key"][indexMap[name]] = "Primary Key";
      sqlTable->setPKey(true);
    }
  }

  sqlTable->setFieldNum(i);
  sqlTable->attrList.append("Name");
  sqlTable->attrList.append("Type");
  sqlTable->attrList.append("NULL");
  sqlTable->attrList.append("Key");
  sqlTable->attrList.append("Default");
  
  return true;
}

bool PgSqlCl::queryEnd(){
  return wait(0);
}


void PgSqlCl::run(){
  PgResult *res;
  for(res = (PgResult*)resultList.first(); res; res = (PgResult*)resultList.next()){
    
    ExecStatusType stat;
  
    PGresult * result = PQexec(con, res->getStmt().latin1());

    stat = PQresultStatus(result);

    
  
    if(!result || stat == PGRES_FATAL_ERROR || stat == PGRES_NONFATAL_ERROR || stat == PGRES_BAD_RESPONSE){
      res->setError(QString(PQresultErrorMessage(result)));
      exit();
    }else if(stat == PGRES_TUPLES_OK || stat == PGRES_COMMAND_OK){
      res->setStmtOk();
      if(stat == PGRES_TUPLES_OK)
        res->setResult(result);
        if(res->isResult())
          isRes = true;
    }
  }
  queryok = true;
  exit();
}
