/***************************************************************************
                          pgsqlcl.h  -  description
                             -------------------
    begin                : Don Mr 20 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PGSQLCL_H
#define PGSQLCL_H

#include <sqlclient.h>
#include <qthread.h>
#include <libpq-fe.h>
/**
  *@author Olaf Gawenda
  */

#define PGSQL_TYPE_NUM 32

static struct SqlType PgSqlTypes[] = {
  //name, leng, lengDec, selfdef, opt00, opt01, opt02, opt03, opt10, opt11, opt12, opt13
  { "",  false, false, "", "", "","", "", "", "", ""},
  {"bigint", false, false, "", "", "","", "", "", "", ""},
  {"bigserial", false, false, "", "", "","", "", "", "", ""},
  {"bit", false, false, "", "", "","", "", "", "", ""},
  {"varbit", true, false, "", "", "","", "", "", "", ""},
  {"bool", false, false, "", "", "","", "", "", "", ""},
  {"box", false, false, "", "", "","", "", "", "", ""},
  {"bytea", false, false, "", "", "","", "", "", "", ""},
  {"char", true, false, "", "", "","", "", "", "", ""},
  {"varchar", true, false, "", "", "","", "", "", "", ""},
  {"cidr", false, false, "", "", "","", "", "", "", ""},
  {"circle", false, false, "", "", "","", "", "", "", ""},
  {"date", false, false, "", "", "","", "", "", "", ""},
  {"float4", false, false, "", "", "","", "", "", "", ""},
  {"float8", false, false, "", "", "","", "", "", "", ""},
  {"inet", false, false, "", "", "","", "", "", "", ""},
  {"smallint", false, false, "", "", "","", "", "", "", ""},
  {"serial", false, false, "", "", "","", "", "", "", ""},
  {"int", false, false, "", "", "","", "", "", "", ""},
  {"interval", true, false, "", "", "","", "", "", "", ""},
  {"line", false, false, "", "", "","", "", "", "", ""},
  {"lseg", false, false, "", "", "","",  "", "", "", ""},
  {"macaddr", false, false, "", "", "","", "", "", "", ""},
  {"mony", false, false, "", "", "","", "", "", "", ""},
  {"decimal", true, false, "", "", "", "","", "", "", ""},
  {"oid", false, false, "", "", "","", "", "", "", ""},
  {"path", false, false, "", "", "","", "", "", "", ""},
  {"point", false, false, "", "", "","", "", "", "", ""},
  {"polygon", false, false, "", "", "","", "", "", "", ""},
  {"text", false, false, "", "", "","",  "", "", "", ""},
  {"time", true, false, "", "without time zone", "with time zone", "", "", "", "", ""},
  {"timestamp", true, false, "", "without time zone", "with time zone", "",  "", "", "", ""}
};
class PgResult : public Result {
public:
   PgResult(const QString & st);
   ~PgResult();
   void setResult(void *result);
   int getColNum();
   int getRowNum();
   QString getColName(int colnum);
   bool nextRow();
   char * getValue(int colnum);
   void setFirstRow();
private:
    PGresult *res;
    int row;
};                               
class PgSqlCl : public SqlClient, QThread  {
public: 
	PgSqlCl();
    PgSqlCl(const QString& dbHost, int dbPort, const QString& dbUser, const QString& dbPw, const QString& dbName);
	~PgSqlCl();
    bool initSqlCl();
    bool useDb(const QString& db = "");
    bool startQuery(QStringList & stList);
    bool runStmt(const QString & st, bool isRes = true);
    bool getDatabases();
    bool getTables();
    bool getTemplates();
    bool makeTable(SqlTable & table);
    bool createDb(const QString & db, const QString & tempDb = "");
    bool queryEnd();
    bool getSqlTable(SqlTable * sqlTable);
    QString limitClause(int offset, int count);

private:
    void run();
    bool connect(const QString & db);
    bool checkCon();
     PGconn *con;
     
     
     
};

#endif
