/***************************************************************************
                          querymap.cpp  -  description
                             -------------------
    begin                : Fre Jun 13 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "querymap.h"
#include <qfile.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qtextstream.h>

QueryMap::QueryMap(){
  xmlFile = "";
  error = "";
}
QueryMap::~QueryMap(){
}
void QueryMap::createRootNode(){
  root.clear();
  domDoc.setContent(QString("<ksqlshellqueries/>"));
  root = domDoc.firstChild().toElement();
}
bool QueryMap::loadFile(){
  if(xmlFile.isEmpty()){
    error = i18n("no query file");
    return false;    
  }
  QFile file(xmlFile);
  
  if(!file.exists()){
    createRootNode();
    saveFile();
    return true;
  }
  
  if(!file.open(IO_ReadOnly)){
    error =  i18n("can't read ") + xmlFile;
    return false;
  }

  int line = 0;
  int col = 0;
  
  if(!domDoc.setContent(&file, &error, &line, &col)){
    error = i18n("can't load ") + xmlFile + i18n(", error: '") + error + i18n("' in line: ") +  QString("%1").arg(line) + " col: " + QString("%1").arg(col);                                           
    file.close();
    return false;
  }
  root = domDoc.firstChild().toElement();
  file.close();
  return true;
}
bool QueryMap::saveFile(){
  //qDebug("xmlFile %s", xmlFile.latin1());
  if(xmlFile.isEmpty()){
    error = i18n("no query file");
    return false;
  }
      
  QFile file(xmlFile);

  if(!file.open(IO_WriteOnly)){
    error =  i18n("can't write ") + xmlFile;
    return false;
  }

  //qDebug(domDoc.toString().latin1());
  
  QTextStream stream (&file);
  domDoc.save(stream,  4);
  
  file.close();
  return true;
}
QStringList QueryMap::getConNameList(){
  return getNameList(root, "connect");
}
QStringList QueryMap::getDbNameList(const QString & conStr){
  QDomNode con = findNode(root, "connect", conStr);
  if(con.isNull())
    return QStringList();

  return getNameList(con, "database");
}
QStringList QueryMap::getQueryNameList(const QString & conStr, const QString & dbStr){
 
  QDomNode con = findNode(root, "connect", conStr);
  if(con.isNull())
    return QStringList();

  QDomNode db = findNode(con, "database", dbStr);
  if(db.isNull())
    return QStringList();

  return getNameList(db, "query");
}
bool QueryMap::getQueryInput(QueryInput * qi){
  QDomNode con = findNode(root, "connect", qi->getConStr());
  if(con.isNull())
    return false;

  QDomNode db = findNode(con, "database", qi->getDbStr());
  if(db.isNull())
    return false;

  QDomNode query = findNode(db, "query", qi->getQueryName());
  if(query.isNull())
    return false;

  QDomNode discr = findNode(query, "discription");
  if(discr.isNull())
    return false;
    
  QDomNode string = findNode(query, "string");
  if(discr.isNull())
    return false;
  
  qi->setQueryText(discr.toElement().text());
  qi->setQuery(string.toElement().text());
  
  return true;
}
bool QueryMap::insertQueryInput(QDomNode & node, QueryInput *qi){
  QDomNode q = findNode(node, "query", qi->getQueryName());
  if(!q.isNull()){
    error = "Query with name: '" + qi->getQueryName() + "' exists!";
    return false;
  }
  
  QDomElement query = domDoc.createElement("query");
  query.setAttribute("name", qi->getQueryName());

  QDomElement discr = domDoc.createElement("discription");
  QDomText dT = domDoc.createTextNode(qi->getQueryText());
  discr.appendChild(dT);

  QDomElement string = domDoc.createElement("string");
  QDomText sT = domDoc.createTextNode(qi->getQuery());
  string.appendChild(sT);

  query.appendChild(discr);
  query.appendChild(string);

  node.appendChild(query);

  return true;
}
bool QueryMap::insertQueryInput(QueryInput *qi){
  QDomNode con = findNode(root, "connect", qi->getConStr());
  if(con.isNull()){
    QDomElement con = domDoc.createElement("connect");
    con.setAttribute("name", qi->getConStr());
    QDomElement db = domDoc.createElement("database");
    db.setAttribute("name", qi->getDbStr());

    if(!insertQueryInput(db, qi))
      return false;
    
    con.appendChild(db);
    root.appendChild(con); 
  }else{
    QDomNode db = findNode(con, "database", qi->getDbStr());
    if(db.isNull()){
      QDomElement db = domDoc.createElement("database");
      db.setAttribute("name", qi->getDbStr());
      
      if(!insertQueryInput(db, qi))
        return false;
      
      con.appendChild(db);  
    }else{
      if(!insertQueryInput(db, qi))
        return false;
    }
  }
  
  if(!saveFile())
    return false;
    
  return true;
}
bool QueryMap::insertCon(const QString & con){
  QDomNode conNode = findNode(root, "connect", con);
  if(!conNode.isNull()){
      error = "Connection with name: '" + con + "' exists!";
      return false;
  }

  QDomElement conE = domDoc.createElement("connect");
  conE.setAttribute("name", con);
  root.appendChild(conE);
  
  if(!saveFile())
    return false;

  return true;
}
bool QueryMap::insertDb(const QString & con, const QString & db){

  QDomNode conNode = findNode(root, "connect", con);
  QDomNode dbNode = findNode(conNode, "database", db);
  if(!dbNode.isNull()){
    error = "Database with name: '" + db + "' exists!";
    return false;
  }

  QDomElement dbE = domDoc.createElement("database");
  dbE.setAttribute("name", db);
  conNode.appendChild(dbE);
  
  if(!saveFile())
    return false;

  return true;
}
bool QueryMap::updateQueryInput(QueryInput *oldQi, QueryInput *newQi, bool name){

  QDomNode con = findNode(root, "connect", oldQi->getConStr());
  QDomNode db = findNode(con, "database", oldQi->getDbStr());

  QDomNode oldQuery;
  
  if(name){
    if(newQi->getQueryName().isEmpty()){
      error = "Name is rquired";
      return false;
    }
    oldQuery = findNode(db, "query", newQi->getQueryName());
    if(!oldQuery.isNull()){
      error = "Query with Name: '" + newQi->getQueryName() + "' exists!";
      return false;
    }
  }
  
  oldQuery = findNode(db, "query", oldQi->getQueryName());
  
  QDomElement newQuery = domDoc.createElement("query");
  newQuery.setAttribute("name", newQi->getQueryName());

  QDomElement discr = domDoc.createElement("discription");
  QDomText dT = domDoc.createTextNode(newQi->getQueryText());
  discr.appendChild(dT);

  QDomElement string = domDoc.createElement("string");
  QDomText sT = domDoc.createTextNode(newQi->getQuery());
  string.appendChild(sT);

  newQuery.appendChild(discr);
  newQuery.appendChild(string);

  QDomNode oQ= db.replaceChild(newQuery, oldQuery);
  if(oQ.isNull()){
    error = "can't update queries";
    return false;
  }
  
  if(!saveFile())
    return false;

  return true;
}
QDomNode QueryMap::findNode(QDomNode & node, const QString & tagName, const QString & name){
  QDomNode out;
  QDomNode n = node.firstChild();

  while(!n.isNull()){
    if(n.isElement()){
      QDomElement e = n.toElement();
      if(e.tagName() == tagName)
        if(name.isEmpty() || e.attribute(QString("name")) == name)
          return n;
    }
    n = n.nextSibling();
  }
  return out;
}
QStringList QueryMap::getNameList(QDomNode & node,  const QString & tagName){
  QStringList list;
  QDomNode n = node.firstChild();
  while(!n.isNull()){
    QDomElement e = n.toElement();
    if(e.tagName() == tagName && e.hasAttribute(QString("name")))
      list.append(e.attribute(QString("name")));
    n = n.nextSibling();
  }

  return list;
}
bool QueryMap::deleteNode(const QString & con, const QString & db, const QString & query){
  QDomNode conN = findNode(root, "connect", con);
  if(db.isEmpty())
    root.removeChild(conN);
  else{
    QDomNode dbN = findNode(conN, "database", db);
    if(query.isEmpty())
      conN.removeChild(dbN);
    else{
      QDomNode queryN = findNode(dbN, "query", query);
      dbN.removeChild(queryN);
    }
  }
   if(!saveFile())
    return false;

  return true;
}
bool QueryMap::rename(const QString &nName, const QString & con, const QString & db, const QString & query){

  //qDebug("new: >%s<, con: >%s<, db: >%s<, query: >%s<", nName.latin1(), con.latin1(), db.latin1(), query.latin1());
  
  if(nName.isEmpty()){
    error = i18n("new Name is required");
    return false;
  }
  
  QDomNode conN = findNode(root, "connect", con);
  if(conN.isNull()){
    error = i18n("connection") + " '" + con + "' " + i18n("don't exist") + "!";
    return false;
  }

  if(db.isEmpty()){
    if(!findNode(root, "connect", nName).isNull()){
      error = i18n("connection")  + " '" + nName + "' " + i18n("exists") + "!";
      return false;
    }
    
    QDomElement conE = conN.toElement();
    conE.setAttribute("name", nName);
    
  }else{
    if(!findNode(conN, "database", nName).isNull()){
      error = i18n("database")  + " '" + nName + "' " + i18n("exists") + "!";
      return false;
    }
    QDomNode dbN = findNode(conN, "database", db);
    if(dbN.isNull()){
      error = i18n("database") + " '" + db + "' " + i18n("don't exist") + "!";
      return false;
    }
    QDomElement dbE;
    if(query.isEmpty()){
      dbE = dbN.toElement();
      dbE.setAttribute("name", nName);
  
    }else{
      if(!findNode(dbN, "query", nName).isNull()){
        error = i18n("query")  + " '" + nName + "' " + i18n("exists") + "!";
        return false;
      }
      QDomNode qN = findNode(dbN, "query", query);
      if(qN.isNull()){
        error = i18n("query") + " '" + query + "' " + i18n("don't exist") + "!";
        return false;
      }
    
      QDomElement qE = qN.toElement();
      qE.setAttribute("name", nName);
    
    }
  }
  
  if(!saveFile())
    return false;
  return true;
}
