/***************************************************************************
                          referencemap.cpp  -  description
                             -------------------
    begin                : Fre Mai 9 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kmessagebox.h>
#include <kiconloader.h>
#include "referencemap.h"

ReferenceMap::ReferenceMap(){
  error = "";
  KIconLoader iLoader;
  errorPix = iLoader.loadIcon("error", KIcon::Small);
  dbPix = iLoader.loadIcon("database",KIcon::Small);
  tabPix = iLoader.loadIcon("table", KIcon::Small);
}
ReferenceMap::~ReferenceMap(){
}
void ReferenceMap::initConnection(Connection & con){
  con.error = "";
  con.valid = false;
  con.sync = true;
  con.databaseMap.clear();
}
void ReferenceMap::initDatabase(Database & db){
  db.sync = false;
  db.valid = false;
  db.error = "";
  db.tableMap.clear();
}
void ReferenceMap::initTable(Table & table){
  table.sync = false;
  table.valid = false;
  table.error = "";
  table.fieldMap.clear();
}
void ReferenceMap::initField(Field & field){
  field.sync = false;
}

void ReferenceMap::insertConnection(const QString & con, SqlClient *sqlCl){
  Connection connection;
  connection.sqlCl = sqlCl;
  initConnection(connection);
  connectionMap[con] = connection; 
}

bool ReferenceMap::contains(const QString & con, const QString &db){
  if(connectionMap.contains(con)){
    if(!db.isEmpty()){
      return connectionMap[con].databaseMap.contains(db);
    }else{
      return true;
    }
  }else{
    return false;
  }  
}
void ReferenceMap::load(){
  start();
}
void ReferenceMap::removeConnection(const QString & conStr){
  if(connectionMap.contains(conStr)){
    delete connectionMap[conStr].sqlCl;
    connectionMap.remove(conStr);
  }
}

void ReferenceMap::sync(QListView *listView){
  QListViewItem  *con, *nextCon, *tmpCon, *db, *nextDb, *tmpDb, *tab, *nextTab, *tmpTab, *field, *nextField, *tmpField;
  con = listView->firstChild();
  while(con){

    db =  con->firstChild();
    while(db){

      tab = db->firstChild();
      while(tab){

        field = tab->firstChild();
        while(field){
          tmpField = field;
          nextField = field->nextSibling();
          field = nextField;
          syncField(tab, tmpField);
        }

        tmpTab = tab;
        nextTab = tab->nextSibling();
        tab = nextTab;
        syncTable(db, tmpTab);
      }

      tmpDb = db;
      nextDb = db->nextSibling();
      db = nextDb;
      syncDatabase(con, tmpDb);
    }
   
    tmpCon = con;
    nextCon = con->nextSibling();
    con = nextCon;
    syncConnection(tmpCon);
  }

}
void ReferenceMap::syncConnection(QListViewItem *conItem){
  if(!conItem)
    return;
  QString conStr = conItem->text(0);
  
  if(connectionMap.contains(conStr)){
    Connection * con = &connectionMap[conStr];
    if(!con->valid){
      conItem->setPixmap(1, errorPix);
      conItem->setText(1, con->error);
    }else{
      conItem->setPixmap(1, 0);                                                    
      conItem->setText(1, "activ");
    }
    
    DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;

    dbMapIt =  dbMap->begin();
    for(;dbMapIt != dbMap->end(); ++dbMapIt)
      if(!dbMapIt.data().sync){
        QListViewItem *dbItem = new QListViewItem(conItem, dbMapIt.key());
        dbItem->setPixmap(0, dbPix);
        syncDatabase(conItem, dbItem);
      }
    
  }else{

    QListViewItem *item, *nItem, *tItem;
    conItem->setPixmap(1, 0);
    conItem->setText(1, "");
    item = conItem->firstChild();
    while(item){
      tItem = item;
      nItem = item->nextSibling();
      item = nItem;
      conItem->takeItem(tItem);
      delete tItem;
    }
  }
}
void ReferenceMap::syncDatabase(QListViewItem *conItem, QListViewItem *dbItem, const QString & dbStr){
  
  if(!dbItem && dbStr.isEmpty())
    return;
  else if(!dbItem){
    dbItem = new QListViewItem(conItem, dbStr); 
    dbItem->setPixmap(0, dbPix);
  }    

  QString dbS = dbItem->text(0);
    
  QString conStr = conItem->text(0);
  
  
  if(!connectionMap.contains(conStr))
    return;

  DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;

  if(dbMap->contains(dbS)){
    Database *db = &(*dbMap)[dbS];
    db->sync = true;

    if(!db->valid){
      dbItem->setPixmap(1, errorPix);
      dbItem->setText(1, db->error);
    }else{
      dbItem->setPixmap(1, 0);
      dbItem->setText(1, "");
    }

    
    TableMap *tableMap = &db->tableMap;
    tableMapIt = tableMap->begin();
    for(; tableMapIt != tableMap->end(); ++tableMapIt)
      if(!tableMapIt.data().sync){
        QListViewItem *tabItem =  new QListViewItem(dbItem, tableMapIt.key(), tableMapIt.data().count);
        tabItem->setPixmap(0, tabPix);
        syncTable(dbItem, tabItem);
      }
    
  }else{
    conItem->takeItem(dbItem);
    delete dbItem;
  }
}
void ReferenceMap::syncTable(QListViewItem *dbItem, QListViewItem *tabItem, const QString & tabStr){
  if(!tabItem && tabStr.isEmpty())
    return;
  else if(!tabItem){
    tabItem = new QListViewItem(dbItem, tabStr);
    tabItem->setPixmap(0, tabPix);
  }


  QListViewItem *conItem = dbItem->parent();
  
  QString conStr = conItem->text(0);
  QString dbStr = dbItem->text(0);
  QString tabS = tabItem->text(0);

  if(!connectionMap.contains(conStr))
    return;

  DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;

  if(!dbMap->contains(dbStr))
    return;

  TableMap * tableMap =  &(*dbMap)[dbStr].tableMap;

  if(tableMap->contains(tabS)){
    Table *table = &(*tableMap)[tabS];
    if(!table->valid){
      tabItem->setPixmap(1, errorPix);
      tabItem->setText(1, table->error);
    }else{
      tabItem->setPixmap(1, 0);
      tabItem->setText(1, table->count);
    }
    
    table->sync = true;
    
    FieldMap * fieldMap = &table->fieldMap;
    fieldMapIt = fieldMap->begin();
    for(; fieldMapIt != fieldMap->end(); ++fieldMapIt) 
      if(!fieldMapIt.data().sync)
        new QListViewItem(tabItem, fieldMapIt.key(), fieldMapIt.data().fieldType);
            
  }else{
    dbItem->takeItem(tabItem);
    delete tabItem;
  }
}
void ReferenceMap::syncField(QListViewItem *tabItem, QListViewItem *fieldItem, const QString & field){
  QString fieldStr = field;
  if(!tabItem)
    return;
  if(!fieldItem && fieldStr.isEmpty()){
    return;
  }else if(!fieldItem){
    fieldItem = new QListViewItem(tabItem, fieldStr);
  }else{
    fieldStr = fieldItem->text(0);
  }

  QListViewItem *conItem, *dbItem;
  dbItem = tabItem->parent();
  conItem = dbItem->parent();
    
  QString conStr = conItem->text(0);
  QString dbStr = dbItem->text(0);
  QString tabStr = tabItem->text(0);

  //qDebug("field: %s" , fieldStr.latin1());
      
  if(!connectionMap.contains(conStr))
    return;

  DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;

  if(!dbMap->contains(dbStr))
    return;

  TableMap * tableMap =  &(*dbMap)[dbStr].tableMap;

  if(!tableMap->contains(tabStr))
    return;
    
  FieldMap *fieldMap = &(*tableMap)[tabStr].fieldMap;

  if(fieldMap->contains(fieldStr)){
    fieldItem->setText(1, (*fieldMap)[fieldStr].fieldType);
    (*fieldMap)[fieldStr].sync = true;
  }else{
    tabItem->takeItem(fieldItem);
    delete fieldItem;
  }
}

bool ReferenceMap::loadEnd(){
  return wait(0);
}
QString ReferenceMap::getError(const QString & conStr, const QString &dbStr){
  if(conStr.isEmpty()){
    QString err = error;
    error = "";
    return err;
  }else if(connectionMap.contains(conStr)){
    if(!dbStr.isEmpty()){
      if(connectionMap[conStr].databaseMap.contains(dbStr)){
        return connectionMap[conStr].databaseMap[dbStr].error;
      }
    }else{
      return connectionMap[conStr].error;
    }
  }
  return QString("");
}
bool ReferenceMap::loadConnection(const QString & conStr){

  if(!connectionMap.contains(conStr)){
    error = "can't find Connection: " + conStr;
    return false;
  }

  Connection *con = &connectionMap[conStr];
  initConnection(*con);

  SqlClient *sqlCl = con->sqlCl;
  sqlCl->clear();
  
  if(sqlCl->getDatabases()){

    Result *res = sqlCl->resultList.at(0);  
    while(res->nextRow())
      con->databaseMap[QString(res->getValue(0))].count = QString("0");
      
    con->valid = true;
  }else{
    con->error = sqlCl->getError();
    return false;
  }

  dbMapIt = con->databaseMap.begin();
  for(; dbMapIt != con->databaseMap.end(); ++dbMapIt)
    loadDatabase(conStr, dbMapIt.key());
    
  return true;
}
bool ReferenceMap::loadDatabase(const QString & conStr, const QString & dbStr){

  if(!connectionMap.contains(conStr)){
    error = "can't find Connection: " + conStr;
    return false;
  }
  DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;
  if(!dbMap->contains(dbStr)){
    error = "can't find Database: " + conStr + " " + dbStr;
    return false;
  }

  Database *db = &(*dbMap)[dbStr];
  initDatabase(*db);

  SqlClient *sqlCl = connectionMap[conStr].sqlCl;
  sqlCl->clear();

  
  
  if(sqlCl->useDb(dbStr) && sqlCl->getTables()){

    Result *res = sqlCl->resultList.at(0);
    while(res->nextRow())
      db->tableMap[QString(res->getValue(0))].count = QString("0");

     db->valid = true;
  }else{
    db->error = sqlCl->getError();
    return false;
  }

  tableMapIt = db->tableMap.begin();
  for(; tableMapIt != db->tableMap.end(); ++tableMapIt)
    loadTable(conStr, dbStr, tableMapIt.key());
  
      
  return true;
}
bool ReferenceMap::loadTable(const QString & conStr, const QString & dbStr, const QString & tabStr){
  
  if(!connectionMap.contains(conStr)){
    error = "can't find Connection: " + conStr;
    return false; 
  }
  DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;
  if(!dbMap->contains(dbStr)){
    error = "can't find Database: " + conStr + " " + dbStr;
    return false;
  }
  TableMap *tableMap = &(*dbMap)[dbStr].tableMap;
  if(!tableMap->contains(tabStr)){
    error = "can't find Table: " + conStr + " " + dbStr + " " + tabStr;
    return false;
  }

  Table *tab = &(*tableMap)[tabStr];
  initTable(*tab);

  SqlClient *sqlCl = connectionMap[conStr].sqlCl;
  sqlCl->clear();

  SqlTable sqlTable;
  sqlTable.setConStr(conStr);
  sqlTable.setDbStr(dbStr);
  sqlTable.setTabStr(tabStr);
  tab->sqlTable = sqlTable;
   
  if(sqlCl->useDb(dbStr) && sqlCl->getSqlTable(&tab->sqlTable)){
    for(int i = 0; i < tab->sqlTable.getFieldNum(); i++){
      Field field;
      initField(field);
      field.fieldType = tab->sqlTable.mapList["Type"][i];
      tab->fieldMap[tab->sqlTable.mapList["Name"][i]] = field;
    }

    tab->valid = true;
  }else{
    tab->error = sqlCl->getError();
    tab->sqlTable.setError(tab->error);
    return false;
  }

  sqlCl->clear();

  QString stmt = "select count(*) from " + tabStr;
  if(!sqlCl->useDb(dbStr) || !sqlCl->runStmt(stmt, true)){
    tab->error = sqlCl->getError();
    return false;
  }
  
  Result *res = sqlCl->resultList.at(0);
    while(res->nextRow())
      tab->count = QString(res->getValue(0));
  
  return true;
}

void ReferenceMap::run(){
  conMapIt = connectionMap.begin();

  for(;conMapIt != connectionMap.end(); ++conMapIt){
    loadConnection(conMapIt.key());
  }
  
}


void ReferenceMap::test(){
  conMapIt = connectionMap.begin();
  for(; conMapIt != connectionMap.end(); ++conMapIt){
    //cout << conMapIt.key() << endl;

    DatabaseMap *dbMap = &conMapIt.data().databaseMap;
    dbMapIt =  dbMap->begin();
    for(;dbMapIt != dbMap->end(); ++dbMapIt){
      //cout << "       " << dbMapIt.key() << endl;

      TableMap *tabMap = &dbMapIt.data().tableMap;
      tableMapIt = tabMap->begin();
      for(;tableMapIt != tabMap->end(); ++tableMapIt){
        //cout << "              " << tableMapIt.key() << endl;

        FieldMap * fMap = &tableMapIt.data().fieldMap;
        fieldMapIt = fMap->begin();
        for(; fieldMapIt != fMap->end(); ++fieldMapIt);
          //cout << "                          " << fieldMapIt.key() << endl;
      }
    }
  }
}
QStringList ReferenceMap::getTables(const QString & conStr, const QString & dbStr){
  QStringList list;
  if(connectionMap.contains(conStr)){
    DatabaseMap *dbMap = &connectionMap[conStr].databaseMap;
    if(dbMap->contains(dbStr)){
      tableMapIt = (*dbMap)[dbStr].tableMap.begin();
      for(; tableMapIt != (*dbMap)[dbStr].tableMap.end(); ++tableMapIt)
        list.append(tableMapIt.key());
    }
  }
    
  return list;
}
SqlTable ReferenceMap::getSqlTable(const QString & conStr, const QString & dbStr, const QString & tabStr){
  SqlTable sqlTable;
  if(connectionMap.contains(conStr)){
    DatabaseMap dbMap = connectionMap[conStr].databaseMap;
    if(dbMap.contains(dbStr)){
      TableMap tabMap = dbMap[dbStr].tableMap;
      if(tabMap.contains(tabStr)){
        return tabMap[tabStr].sqlTable;
      }
    }
  }
  return  sqlTable;
}
bool ReferenceMap::isValid(const QString & conStr, const QString &dbStr){
  if(connectionMap.contains(conStr)){
    if(!dbStr.isEmpty()){
      if(connectionMap[conStr].databaseMap.contains(dbStr)){
        return  connectionMap[conStr].databaseMap[dbStr].valid;
      }
    }else{
      return connectionMap[conStr].valid;
    }
  }
  return false;
}


