/***************************************************************************
                          referencemap.h  -  description
                             -------------------
    begin                : Fre Mai 9 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REFERENCEMAP_H
#define REFERENCEMAP_H

#include <qstring.h>
#include <qmap.h>
#include <qvaluevector.h>
#include <qlistview.h>
#include <qstringlist.h>
#include <qthread.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include "sqlclient.h"
#include "sqltable.h"
/**
  *@author Olaf Gawenda
  */
class ReferenceMap : public QThread{
public:

  typedef struct {
      QString fieldType;
      bool sync;
  } Field;

  typedef QMap<QString,  Field> FieldMap;

  typedef struct {
      FieldMap fieldMap;
      QString count;
      bool sync;
      QString error;
      bool valid;
      SqlTable sqlTable;
  } Table;

  typedef QMap<QString, Table> TableMap;

  typedef struct {
      TableMap tableMap;
      QString count;
      bool sync;
      QString error;
      bool valid;
  } Database;

  typedef QMap<QString, Database> DatabaseMap;

  typedef struct {
      DatabaseMap databaseMap;
      QString label;
      SqlClient *sqlCl;
      bool sync;
      QString error;
      bool valid;
  } Connection;
  
	ReferenceMap();
	~ReferenceMap();

  void initConnection(Connection & con);
  void initDatabase(Database & db);
  void initTable(Table & table);
  void initField(Field & field);
  void insertConnection(const QString & con, SqlClient *sqlCl);
  bool contains(const QString & con, const QString & db = "");
  void load();
  bool loadConnection(const QString & conStr);
  bool loadDatabase(const QString & conStr, const QString & dbStr);
  bool loadTable(const QString & conStr, const QString & dbStr, const QString & tabStr);
  void removeConnection(const QString & conStr);
  void sync(QListView *listView);
  void syncConnection(QListViewItem *conItem);
  void syncDatabase(QListViewItem *conItem, QListViewItem *dbItem, const QString & dbStr = "");
  void syncTable(QListViewItem *dbItem, QListViewItem *tabItem, const QString & tabStr = "");
  void syncField(QListViewItem *tabItem, QListViewItem *fieldItem, const QString & field = "");
  bool loadEnd();
  QString getError(const QString & conStr = "", const QString &dbStr = "");
  bool isValid(const QString & conStr, const QString &dbStr = "");
  void test();

  QStringList getTables(const QString & conStr, const QString & dbStr);
  SqlTable getSqlTable(const QString & conStr, const QString & dbStr, const QString & tabStr);

private:
  void run();
  QMap<QString, Connection> connectionMap;                         
  FieldMap::iterator fieldMapIt;
  TableMap::iterator tableMapIt;
  DatabaseMap::iterator dbMapIt;
  QMap<QString, Connection>::iterator conMapIt;
  QString error;
  QPixmap dbPix, tabPix, errorPix;
};

#endif
