/***************************************************************************
                          shell.cpp  -  description
                             -------------------
    begin                : Mit Mr 5 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qregexp.h>
#include <klocale.h>
#include "shell.h"

Shell::Shell(KSqlShellDoc *document, QWidget *parent, const char *name ) : QTextEdit(parent,name) {
  doc = document;
  //connect(this, SIGNAL(returnPressed()),this, SLOT(slotReturn()));
  connect(this, SIGNAL(cursorPositionChanged(int ,int)), this, SLOT(slotCursorPositionChanged(int , int)));
  shellData = 0;
  error = "";
  setTextFormat(PlainText);
  if(parent)
    parent->installEventFilter(this);
}
Shell::~Shell(){
}
QString Shell::getCurrentConStr(){
  if(shellData)
    return shellData->con;
  return QString("");
}
QString Shell::getCurrentDbStr(){
  if(shellData)
    return shellData->db;
  return QString("");
}
QString Shell::getCurrentQuery(){
  QString queryText = "";
  
  int para = paragraphs() - 1;
  if(para < 0)
    return queryText;

  for(int i = shellData->lastCmdPara; i <= para; i++ ){
    queryText.append(text(i));
    queryText.append('\n');
  }
  
  queryText.replace(QRegExp(shellData->prompt), "");
  queryText.remove(queryText.length() - 2, 2);
  
  return queryText;
}
void Shell::getCurrentQueryInput(QueryInput *qi){
  qi->setConStr(getCurrentConStr());
  qi->setDbStr(getCurrentDbStr());
  qi->setQuery(getCurrentQuery());
}

bool Shell::eventFilter(QObject *o, QEvent *e){
  if(e->type() == QEvent::KeyPress){
    QKeyEvent *k = (QKeyEvent *)e;

    //qDebug("%d", k->key());
                
    int para, index;
    getCursorPosition(&para, &index);

    if(k->key() == 4099 && para == shellData->lastCmdPara && index <= (int)shellData->promptLen){
      return true;
    }
    if(k->key() == 4100){
      slotReturn();
      return true;  
    }
    if(k->key() == 4115){
      if(para <= shellData->lastCmdPara){
        history();
        return true;
      }
    }else if(k->key() == 4117){
      if(para >= paragraphs() -1){
        history(true);
        return true;
      }  
    }
  }
  return QTextEdit::eventFilter(o, e);
}
void Shell::slotChangeDb(const QString & conStr, const QString &dbStr){
  QString noCon = i18n("is not connected!");
  
  if(shellData && !text().isEmpty()){
    shellData->text = text();
    shellData->text.replace(QRegExp("\n" + noCon), "");
  }
    
  setReadOnly(true);
  
  if(conStr.isEmpty() || !doc->clQueueMap.contains(conStr)){
    clear();  
    return;
  }
  
  QString db = dbStr;
  
  if(dbStr.isEmpty())
    db = doc->clQueueMap[conStr]->getDefDb();
  
  clear();
  
  if(!shellDatas.contains(conStr, db))
    shellDatas.newData(conStr, db);
  
  shellData = shellDatas.getShellData(conStr, db);

  setReadOnly(false);
  
  if(!shellData->text.isEmpty()){
    insert(shellData->text);
  }else
    insert(shellData->prompt);

  if(!doc->clQueueMap[conStr]->isConnect()){
    
    append(noCon);
    setReadOnly(true);
    
    return;
  }
  
   
  if(doc->refMap->contains(conStr)){
    if(!doc->refMap->isValid(conStr)){
      setReadOnly(true);
      append(doc->refMap->getError(conStr));
      return;
    }
    
    if(doc->refMap->contains(conStr, db)){
      if(!doc->refMap->isValid(conStr, db)){
        setReadOnly(true);
        append(doc->refMap->getError(conStr, db));
        return;
      }
    }   
  }
}

void Shell::slotReturn(){
  
  int para = paragraphs() - 1;
  if(para < 0)
    return;
    
  QString cmdText = "";

  for(int i = shellData->lastCmdPara; i <= para; i++ ){
    cmdText.append(text(i));
  }                                                  

  QString cmd = cmdText.stripWhiteSpace();
  
  //qDebug("cmd: >%s<", cmd.latin1());
  
  uint cmdEnd = cmd.length() - 1;

  QString str(cmd.at(cmdEnd));
  //qDebug("end: >%s<" , str.latin1());
  
  if(cmd.at(cmdEnd) == ';'){
    
    setCursorPosition(para , cmd.length());
    
    cmd.replace(QRegExp(shellData->prompt), "");
    
    if(!runInput(cmd))
      insert(getError());

    insert("\n");  
    insert(shellData->prompt);
    shellData->lastCmdPara = para + 1;  
  }else{
    insert("\n");
  }
}
void Shell::slotCursorPositionChanged(int para, int pos){
                                                   
  if(!shellData || isReadOnly())
    return;
  
  if(para < shellData->lastCmdPara || para == shellData->lastCmdPara && pos < (int)shellData->promptLen)
    setCursorPosition( shellData->lastCmdPara, shellData->promptLen);

}
bool Shell::runInput(const QString & input){

  if(input.isEmpty())
    return true;

  QString *histStr = new QString(input);
  shellData->history.prepend(histStr);

  shellData->histIndex = 0;
  
  if(shellData->history.count() > 1000){
    QString *last = shellData->history.last();
    shellData->history.removeLast();
    delete last;
  }
  
  if(!sendQuery(input))
    return false;
  
  return true;
}
bool Shell::sendQuery(const QString & query){

  QueryInput qi;

  qi.setDbStr(shellData->db);
  qi.setConStr(shellData->con);
  qi.setType(QueryInput::Free);
  qi.setQuery(query);

  if(!doc->sendQuery(qi)){
    error = doc->getError();
    return false;
  }
  
  return true;
}
void Shell::history(bool back){
  
  if(shellData->history.count() > shellData->histIndex){
    QString *histStr = shellData->history.at(shellData->histIndex);
    if(histStr){                                                                                               
      int i = paragraphs();
      int para = shellData->lastCmdPara;
    
      for(;i >= para; i -- ){
        removeParagraph(i);
      }
    
      append(shellData->prompt);
      setCursorPosition(paragraphs() -1, shellData->promptLen);
      insert(*histStr);

      if(back){
        shellData->histIndex --;
      }else{
        shellData->histIndex ++;
      }
    }
  }else{
    if(back){
      shellData->histIndex = shellData->history.count() - 1;
    }else{
      shellData->histIndex = 0;
    }
  }
}

QString Shell::getError(){
  QString err = error;
  error = "";
  return err;
}
