/***************************************************************************
                          sqlclient.h  -  description
                             -------------------
    begin                : Mon Mr 10 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQLCLIENT_H
#define SQLCLIENT_H

#include <qstring.h>
#include <qstringlist.h>
#include <qptrlist.h>
#include <qmap.h>
#include "sqltable.h"
#include "queryinput.h"

/**
  *@author Olaf Gawenda
  */

struct SqlType{
  QString name;
  bool lengDec;
  bool selfDef;
  QString opt00;
  QString opt01;
  QString opt02;
  QString opt03;
  QString opt10;
  QString opt11;
  QString opt12;
  QString opt13;
};

class SqlTypes{
  public:
    SqlTypes();
    SqlTypes( SqlType *sqlType, int len);
    ~SqlTypes();
    QStringList typeList;
    QMap<QString, QStringList> opt0Map, opt1Map;
    QMap<QString, SqlType> sqlTypeMap;
};
class Result {
  public:
    Result(const QString & st);
    virtual ~Result();
    virtual void setResult(void *result) = 0;
    void setTable(const QString &tab) { table = tab; };
    QString getTable() { return table; };
    void setError(const QString & err) { error = err; };
    QString getError() { return error; };
    QString getStmt(){ return stmt; };
    void setStmtOk() {stmtok = true; };
    bool isResult() { return isRes; };
    bool stmtOk() { return stmtok; };
    
    virtual int getColNum() = 0;
    virtual int getRowNum() = 0;
    virtual QString getColName(int colnum) = 0;
    virtual bool nextRow() = 0;
    virtual char * getValue(int colnum) = 0;
    virtual void setFirstRow() = 0;
    
protected:
    QString stmt;
    QString table;
    QString error;
    bool stmtok;
    bool isRes;
};
class SqlClient {
public:
    
    SqlClient(); 
	SqlClient(const QString& dbHost, int dbPort, const QString& dbUser, const QString& dbPw, const QString& dbName);
	virtual ~SqlClient();
    virtual bool initSqlCl() = 0;
    virtual bool useDb(const QString& db = "") = 0;
    virtual bool startQuery(QStringList & stList) = 0;
    bool sendQuery(const QueryInput & qI);
    virtual bool getDatabases() = 0;
    virtual bool getTables() = 0;
    bool getVersionStr();
    float getVersionNum(const QString & versionStr);
    virtual bool getSqlTable(SqlTable * sqlTable) = 0;
    void setCurrentTable(const QString &t){ table = t; };
    virtual bool getTemplates() = 0;
    virtual QString getCurrentDb();
    virtual bool makeTable(SqlTable & table) = 0;
    virtual bool createDb(const QString & db, const QString & tempDb) = 0;
    virtual bool runStmt(const QString  & st, bool isRes = true) = 0;
    virtual bool queryEnd() = 0;
    virtual bool queryOk();
    virtual QString getError();
    void clear();
    bool isResult() { return isRes; };
    virtual QString limitClause(int offset, int count) = 0;
    QueryInput queryInput;
    QPtrList <Result> resultList;
     
protected:
    
    QString host;
    QString user;
    QString pw;
    QString defDb;
    QString aktDb;
    QString error;
    QString table;
    bool queryok;
    bool isRes;
    int port;
    
};

#endif
