/***************************************************************************
                          tabdlg.cpp  -  description
                             -------------------
    begin                : Mon Apr 7 2003
    copyright            : (C) 2003 by Olaf Gawenda
    email                : o.gawenda@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qsize.h>
#include <qrect.h>
#include <klocale.h>
#include <qcombobox.h>
#include "tabdlg.h"


TabDlg::TabDlg(QWidget *parent, const QString & caption, KSqlShellDoc * document, const QString & conStr, const QString & database)
                            :KDialogBase(parent, "tabdlg", true, caption, Ok | User1| Cancel, Ok, true)
{
  setButtonText(User1, i18n("edit create statement"));
  edit = false;
  doc = document;
  con = conStr;
  db = database;
  colPos = 0;

  page = new QWidget(this);
  setMainWidget(page);

  layout = new QVBoxLayout(page, 0, spacingHint());
  hLayout = new QHBoxLayout(layout, 0);

  vGroupBox = new QVGroupBox(i18n("Table Name: "), page);

  tableName = new QLineEdit(vGroupBox);
  table = new Table(0, 8, page);

  hLayout->addWidget(vGroupBox);

  layout->addSpacing(20);
  
  layout->addWidget(table);
  
  createText = new QTextEdit(page);
  layout->addWidget(createText);
  createText->setEnabled(false);
  
  menu = new QPopupMenu();
  menu->insertItem(i18n("add column"), 0);
  menu->insertItem(i18n("del column"), 1);

  
  table->horizontalHeader()->setLabel(0,i18n("Name"));
  table->horizontalHeader()->setLabel(1,i18n("Type"));
  table->horizontalHeader()->setLabel(2,i18n("Length, Dec"));
  table->horizontalHeader()->setLabel(3,i18n("Attribute"));
  table->horizontalHeader()->setLabel(4, i18n("Default Value"));
  table->horizontalHeader()->setLabel(5,i18n("Not Null"));
  table->horizontalHeader()->setLabel(6,i18n("Unique"));
  table->horizontalHeader()->setLabel(7,i18n("Primary Key"));

  table->setColumnWidth(1, 100);
  table->setColumnWidth(2, 80);
  table->setColumnWidth(3, 100);
  table->setColumnWidth(4, 100);
  table->setColumnWidth(5, 55);
  table->setColumnWidth(6, 55);
  table->setColumnWidth(7, 80);
  
  table->setSelectionMode(QTable::NoSelection);

  sqlTypes = doc->clQueueMap[con]->getSqlTypes();
  
    
  connect(table->verticalHeader(), SIGNAL(clicked(int)), this, SLOT(slotMenu(int)));
  connect(menu, SIGNAL(activated(int)), this, SLOT(slotAddDelCol(int)));
  connect(table, SIGNAL(valueChanged(int , int )), this, SLOT(slotValueChanged(int,int)));

}
TabDlg::~TabDlg(){
}
void TabDlg::startDlg(){
  initDlg();
  exec();
}
void TabDlg::initDlg(){
  for(colPos = -1; colPos < 3; colPos++)
    slotAddCol();
}
void TabDlg::slotMenu(int pos){
    colPos = pos;
    QRect rect = table->verticalHeader()->sectionRect(pos);
    
    if(table->numRows() > 1)
      menu->setItemEnabled(1, true);
    else                                             
      menu->setItemEnabled(1, false);
      
    menu->popup( table->verticalHeader()->mapToGlobal(rect.topLeft()) += QPoint(25, 8));
}
void TabDlg::slotAddCol(){
    int pos = colPos + 1;
    
    table->insertRows(pos);

    //Name
    table->setItem(pos, 0, new QTableItem(table, QTableItem::OnTyping, ""));
    //Type
    table->setItem(pos, 1, new QComboTableItem(table, sqlTypes.typeList, true));
    //Length / Dec
    table->setItem(pos, 2, new QTableItem(table, QTableItem::OnTyping, ""));
    //attr
    table->setItem(pos, 3, new QComboTableItem(table,  0, true));
    //default value
    table->setItem(pos, 4, new QTableItem(table, QTableItem::OnTyping, ""));
    //not null
    table->setItem(pos, 5, new QCheckTableItem(table,""));
    //unique
    table->setItem(pos, 6, new QCheckTableItem(table,""));
    //primary key
    table->setItem(pos, 7, new QCheckTableItem(table,""));

    table->item(pos, 2)->setEnabled(false);

    addCol(pos);
    
}
void TabDlg::slotDelCol(){
    table->removeRow(colPos);
}
void TabDlg::slotAddDelCol(int id){
  if(id == 0)
    slotAddCol();
  else if(id == 1)
    slotDelCol();
}
void TabDlg::slotValueChanged(int row, int col){

  if(col == 1){
    QComboTableItem *typeBox = (QComboTableItem*)table->item(row,col);
    QString type = typeBox->currentText();
    
    ((QComboTableItem*)table->item(row, 3))->setStringList(sqlTypes.opt0Map[type]);

    table->item(row, 2)->setEnabled(sqlTypes.sqlTypeMap[type].lengDec);
    table->setText(row, 2, "");
    table->setText(row, 4, "");
    
    valueChanged(row, col);
  }
}
QStringList TabDlg::getAttrList(int attr){
  QStringList list;
  for(int j = 0; j < table->numRows(); j++)
        list.append(table->item(j, attr)->text());

  return list;
}
QStringList TabDlg::getAttrList(int attr, const QString & attrStr){
  QStringList list;
  for(int j = 0; j < table->numRows(); j++){
    if(((QCheckTableItem*)table->item(j, attr))->isChecked()){
      list.append(attrStr);
    }else{
      list.append("");
    }
  }
  return list;
}

SqlTable TabDlg::getValues(){
  SqlTable sqlTable;

  sqlTable.setTableAction(SqlTable::NewTable);
  sqlTable.setConStr(con);
  sqlTable.setDbStr(db);
  sqlTable.setTabStr(tableName->text());

  sqlTable.setFieldNum(table->numRows());

  sqlTable.mapList["Name"] = getAttrList(0);
  sqlTable.mapList["Type"] = getAttrList(1);
  sqlTable.mapList["Length, Dec" ] = getAttrList(2);
  sqlTable.mapList["Attribute"] = getAttrList(3);
  sqlTable.mapList["Default Value"] = getAttrList(4);
  sqlTable.mapList["Not Null"] = getAttrList(5, "Not Null");
  sqlTable.mapList["Unique"] = getAttrList(6, "Unique");
  sqlTable.mapList["Primary Key"] = getAttrList(7, "Primary Key");

  ok(sqlTable);

  return sqlTable;
}
void TabDlg::slotUser1(){
  if(edit){
    edit = false;
    setButtonText(User1, i18n("edit create statement"));
    setButtonOKText("OK");
    createText->clear();
    createText->setEnabled(false);
  }else{
    table->editEnd();
    SqlTable sqlTable = getValues();
    edit = true;
    setButtonText(User1, i18n("no edit create statement"));
    setButtonOKText(i18n("run create statement"));
    createText->setEnabled(true);
    createText->setText(doc->clQueueMap[con]->getCreateStmt(sqlTable));
  }
  repaint();
}
void TabDlg::slotOk(){

  if(!edit){

    table->editEnd();
    
    if(tableName->text().isEmpty()){
      KMessageBox::sorry(this, i18n("Table name is required"));
      return;
    }
  
    SqlTable sqlTable = getValues();

    if(!doc->makeTable(sqlTable)){
      KMessageBox::error(this, doc->getError());
    }else{
      delayedDestruct();
    }
  }else{
    if(!doc->clQueueMap[con]->runStmt(createText->text(), db)){
      KMessageBox::error(this, doc-> clQueueMap[con]->getError());
    }else{
      delayedDestruct();
    }
  }
}
